@@ -42,12 +42,15 @@ check_custom() {
 }
 
 fail() {
-    pkgname=$1
-		rm -f config.mk
-		echo ""
-		echo "You need package ${pkgname} in order to compile this program."
-		echo "Please make sure it is installed."
-		exit 1
+	pkgname=$1
+	rm -f config.mk
+	dlurl=`grep -i "$pkgname" README | awk '{ print $NF }'`
+	echo ""
+	echo "You need package ${pkgname} in order to compile this program."
+	echo "Please make sure it is installed."
+	echo ""
+	echo "You can download ${pkgname} from here: ${dlurl}"
+	exit 1
 }
 
 fail_custom() {
@@ -61,3 +64,4 @@ echo "" > config.mk
 
 check_pkg "sqlite3" || fail "sqlite3"
 check_pkg "libcurl" || check_custom "libcurl" "curl-config" || fail "libcurl"
+check_pkg "libxml-2.0" || check_custom "libxml2" "xml2-config" || fail "libxml2"
