@@ -3,6 +3,9 @@
 #if !defined(COCO_PARSER_H__)
 #define COCO_PARSER_H__
 
+#include <vector>
+#include <string>
+
 #include "FilterParser.h"
 
 
@@ -13,6 +16,7 @@
 class Errors {
 public:
 	int count;			// number of errors detected
+	std::vector<std::wstring> errors;
 
 	Errors();
 	void SynErr(int n);
@@ -24,14 +28,13 @@ public:
 
 class Parser {
 private:
-	enum {
-		_EOF=0,
-		_openblock=1,
-		_closeblock=2,
-		_ident=3,
-		_stringliteral=4,
-		_numliteral=5,
-	};
+	int _EOF;
+	int _openblock;
+	int _closeblock;
+	int _ident;
+	int _stringliteral;
+	int _numliteral;
+	int _rangeliteral;
 	int maxT;
 
 	Token *dummyToken;
