@@ -4,12 +4,17 @@
 #include <urlreader.h>
 #include <rss.h>
 #include <cache.h>
-#include <_nxml.h>
 #include <configcontainer.h>
 #include <filtercontainer.h>
+#include <colormanager.h>
+#include <regexmanager.h>
+#include <libxml/tree.h>
 
 namespace newsbeuter {
 
+	extern int ctrl_c_hit;
+	extern std::string lock_file;
+
 	class view;
 
 	class controller {
@@ -32,7 +37,7 @@ namespace newsbeuter {
 			std::vector<std::tr1::shared_ptr<rss_item> > search_for_items(const std::string& query, const std::string& feedurl);
 			inline unsigned int get_feedcount() { return feeds.size(); }
 
-			inline void unlock_reload_mutex() { reload_mutex->unlock(); }
+			inline void unlock_reload_mutex() { reload_mutex.unlock(); }
 			bool trylock_reload_mutex();
 
 			void update_feedlist();
@@ -44,6 +49,7 @@ namespace newsbeuter {
 			bool is_valid_podcast_type(const std::string& mimetype);
 			void enqueue_url(const std::string& url, std::tr1::shared_ptr<rss_feed> feed);
 			void notify(const std::string& msg);
+			unsigned int get_pos_of_next_unread(unsigned int pos);
 
 			void reload_urls_file();
 			void edit_urls_file();
@@ -58,18 +64,27 @@ namespace newsbeuter {
 
 			inline cache * get_cache() { return rsscache; }
 
-			inline configcontainer * get_cfg() const { return cfg; }
+			inline configcontainer * get_cfg() { return &cfg; }
 
 			void write_item(std::tr1::shared_ptr<rss_item> item, const std::string& filename);
+			void write_item(std::tr1::shared_ptr<rss_item> item, std::ostream& ostr);
+			std::string write_temporary_item(std::tr1::shared_ptr<rss_item> item);
 
 			void mark_deleted(const std::string& guid, bool b);
 
+			void update_config();
+
+			void load_configfile(const std::string& filename);
+
+			void dump_config(const std::string& filename);
+
+			void sort_feeds();
 		private:
 			void usage(char * argv0);
-			void version_information();
+			void version_information(const char * argv0, unsigned int level);
 			void import_opml(const char * filename);
 			void export_opml();
-			void rec_find_rss_outlines(nxml_data_t * node, std::string tag);
+			void rec_find_rss_outlines(xmlNode * node, std::string tag);
 			void compute_unread_numbers(unsigned int&, unsigned int& );
 			void execute_commands(char ** argv, unsigned int i);
 
@@ -83,8 +98,6 @@ namespace newsbeuter {
 			void import_read_information(const std::string& readinfofile);
 			void export_read_information(const std::string& readinfofile);
 
-			void sort_feeds();
-
 			view * v;
 			urlreader * urlcfg;
 			cache * rsscache;
@@ -95,11 +108,14 @@ namespace newsbeuter {
 			std::string config_file;
 			std::string queue_file;
 			bool refresh_on_start;
-			configcontainer * cfg;
+			configcontainer cfg;
 			rss_ignores ign;
 			filtercontainer filters;
 
-			mutex * reload_mutex;
+			mutex reload_mutex;
+			configparser cfgparser;
+			colormanager colorman;
+			regexmanager rxman;
 	};
 
 }
