@@ -26,8 +26,8 @@ class formaction {
 		virtual ~formaction();
 		virtual void prepare() = 0;
 		virtual void init() = 0;
-		stfl::form * get_form();
-		inline void set_redraw(bool b) { do_redraw = b; }
+		std::tr1::shared_ptr<stfl::form> get_form();
+		virtual void set_redraw(bool b) { do_redraw = b; }
 
 		virtual keymap_hint_entry * get_keymap_hint() = 0;
 
@@ -48,6 +48,16 @@ class formaction {
 		inline std::string get_qna_response(unsigned int i) { return (qna_responses.size() >= (i + 1)) ? qna_responses[i] : ""; }
 		void start_qna(const std::vector<qna_pair>& prompts, operation finish_op, history * h = NULL);
 
+		inline void set_parent_formaction(std::tr1::shared_ptr<formaction> fa) { parent_formaction = fa; }
+		inline std::tr1::shared_ptr<formaction> get_parent_formaction() { return parent_formaction; }
+
+		virtual std::string title() = 0;
+		
+		virtual std::vector<std::string> get_suggestions(const std::string& fragment);
+
+		static void load_histories(const std::string& searchfile, const std::string& cmdlinefile);
+		static void save_histories(const std::string& searchfile, const std::string& cmdlinefile, unsigned int limit);
+
 	protected:
 		virtual void process_operation(operation op, bool automatic = false, std::vector<std::string> * args = NULL) = 0;
 		virtual void set_keymap_hints();
@@ -56,7 +66,7 @@ class formaction {
 		void start_bookmark_qna(const std::string& default_title, const std::string& default_url, const std::string& default_desc);
 
 		view * v;
-		stfl::form * f;
+		std::tr1::shared_ptr<stfl::form> f;
 		bool do_redraw;
 
 		std::vector<std::string> qna_responses;
@@ -64,6 +74,8 @@ class formaction {
 		static history searchhistory;
 		static history cmdlinehistory;
 
+		std::vector<std::string> valid_cmds;
+
 	private:
 		std::string prepare_keymap_hint(keymap_hint_entry * hints);
 		void start_next_question();
@@ -71,6 +83,7 @@ class formaction {
 		std::vector<qna_pair> qna_prompts;
 		operation finish_operation;
 		history * qna_history;
+		std::tr1::shared_ptr<formaction> parent_formaction;
 };
 
 
