@@ -21,9 +21,10 @@ enum {
 	KM_FILTERSELECT	= 1<<6,
 	KM_URLVIEW		= 1<<7,
 	KM_PODBEUTER	= 1<<8,
-	KM_NEWSBEUTER = KM_FEEDLIST | KM_FILEBROWSER | KM_HELP | KM_ARTICLELIST | KM_ARTICLE | KM_TAGSELECT | KM_FILTERSELECT | KM_URLVIEW,
 	KM_SYSKEYS		= 1<<9,
 	KM_INTERNAL		= 1<<10,
+	KM_DIALOGS		= 1<<11,
+	KM_NEWSBEUTER	= KM_FEEDLIST | KM_FILEBROWSER | KM_HELP | KM_ARTICLELIST | KM_ARTICLE | KM_TAGSELECT | KM_FILTERSELECT | KM_URLVIEW | KM_DIALOGS,
 	KM_BOTH			= KM_NEWSBEUTER | KM_PODBEUTER
 };
 
@@ -65,10 +66,18 @@ namespace newsbeuter {
 						OP_DELETE,
 						OP_PURGE_DELETED,
 						OP_EDIT_URLS,
+						OP_CLOSEDIALOG,
+						OP_VIEWDIALOGS,
+						OP_NEXTDIALOG,
+						OP_PREVDIALOG,
+						OP_PIPE_TO,
+						OP_RANDOMUNREAD,
+						OP_SORT,
+						OP_REVSORT,
 						OP_NB_MAX,
 
 						// podbeuter-specific operations:
-						OP_PB_MIN, 
+						OP_PB_MIN = 1000, 
 						OP_PB_DOWNLOAD, 
 						OP_PB_CANCEL, 
 						OP_PB_DELETE,
@@ -79,7 +88,7 @@ namespace newsbeuter {
 						OP_PB_PLAY, 
 						OP_PB_MAX,
 						
-						OP_SK_MIN,
+						OP_SK_MIN = 1500,
 						OP_SK_UP,
 						OP_SK_DOWN,
 						OP_SK_PGUP,
@@ -87,7 +96,7 @@ namespace newsbeuter {
 						/* TODO: add more user-defined keys here */
 						OP_SK_MAX,
 
-						OP_INT_MIN,
+						OP_INT_MIN = 2000,
 
 						OP_INT_END_CMDLINE,
 						OP_INT_END_SETFILTER,
@@ -104,7 +113,7 @@ namespace newsbeuter {
 						OP_INT_SET,
 
 						OP_INT_MAX,
-						OP_1,
+						OP_1 = 3001,
 						OP_2,
 						OP_3,
 						OP_4,
@@ -121,6 +130,7 @@ namespace newsbeuter {
 		std::string cmd;
 		std::string desc;
 		std::string ctx;
+		unsigned short flags;
 	};
 
 	struct macrocmd {
@@ -139,11 +149,13 @@ namespace newsbeuter {
 			std::vector<macrocmd> get_macro(const std::string& key);
 			char get_key(const std::string& keycode);
 			std::string getkey(operation op, const std::string& context);
-			virtual action_handler_status handle_action(const std::string& action, const std::vector<std::string>& params);
+			virtual void handle_action(const std::string& action, const std::vector<std::string>& params);
+			virtual void dump_config(std::vector<std::string>& config_output);
 			void get_keymap_descriptions(std::vector<keymap_desc>& descs, unsigned short flags);
 			unsigned short get_flag_from_context(const std::string& context);
 		private:
 			bool is_valid_context(const std::string& context);
+			std::string getopname(operation op);
 			std::map<std::string, std::map<std::string, operation> > keymap_;
 			std::map<std::string,std::vector<macrocmd> > macros_;
 	};
