@@ -6,13 +6,10 @@
 
 #include <configcontainer.h>
 #include <matcher.h>
+#include <mutex.h>
 
 #include <tr1/memory>
 
-extern "C" {
-#include <_mrss.h>
-}
-
 namespace newsbeuter {
 
 	typedef std::pair<std::string, matcher *> feedurl_expr_pair;
@@ -85,6 +82,9 @@ namespace newsbeuter {
 			inline bool deleted() const { return deleted_; }
 			inline void set_deleted(bool b) { deleted_ = b; }
 
+			inline void set_index(unsigned int i) { idx = i; }
+			inline unsigned int get_index() { return idx; }
+
 		private:
 			std::string title_;
 			std::string link_;
@@ -101,6 +101,7 @@ namespace newsbeuter {
 			std::string flags_;
 			std::tr1::shared_ptr<rss_feed> feedptr;
 			bool deleted_;
+			unsigned int idx;
 	};
 
 	class rss_feed : public matchable {
@@ -129,7 +130,8 @@ namespace newsbeuter {
 			inline const std::string& rssurl() const { return rssurl_; }
 			void set_rssurl(const std::string& u);
 			
-			unsigned int unread_item_count() const;
+			unsigned int unread_item_count();
+			inline unsigned int total_item_count() const { return items_.size(); }
 
 			void set_tags(const std::vector<std::string>& tags);
 			bool matches_tag(const std::string& tag);
@@ -147,6 +149,7 @@ namespace newsbeuter {
 			void set_empty(bool t) { empty = t; }
 
 			void sort(const std::string& method);
+			void sort_unlocked(const std::string& method);
 
 			void remove_old_deleted_items();
 
@@ -155,6 +158,15 @@ namespace newsbeuter {
 			inline void set_rtl(bool b) { is_rtl_ = b; }
 			inline bool is_rtl() { return is_rtl_; }
 
+			inline void set_index(unsigned int i) { idx = i; }
+			inline unsigned int get_index() { return idx; }
+
+			inline void set_order(unsigned int x) { order = x; }
+			inline unsigned int get_order() { return order; }
+
+			void set_feedptrs(std::tr1::shared_ptr<rss_feed> self);
+
+			mutex item_mutex; // this is ugly, but makes it possible to lock items use e.g. from the cache class
 		private:
 			std::string title_;
 			std::string description_;
@@ -169,13 +181,16 @@ namespace newsbeuter {
 
 			bool empty;
 			bool is_rtl_;
+			unsigned int idx;
+			unsigned int order;
 	};
 
 	class rss_ignores : public config_action_handler {
 		public:
 			rss_ignores() { }
 			virtual ~rss_ignores();
-			virtual action_handler_status handle_action(const std::string& action, const std::vector<std::string>& params);
+			virtual void handle_action(const std::string& action, const std::vector<std::string>& params);
+			virtual void dump_config(std::vector<std::string>& config_output);
 			bool matches(rss_item * item);
 			bool matches_lastmodified(const std::string& url);
 			bool matches_resetunread(const std::string& url);
