@@ -6,6 +6,7 @@
 
 #include <logger.h>
 #include <rss.h>
+#include <curl/curl.h>
 
 namespace newsbeuter {
 
@@ -25,7 +26,7 @@ class utils {
 
 		static std::string get_command_output(const std::string& cmd);
 		static void extract_filter(const std::string& line, std::string& filter, std::string& url);
-		static std::string retrieve_url(const std::string& url, const char * user_agent = 0, const char * auth = 0, int download_timeout = 30);
+		static std::string retrieve_url(const std::string& url, configcontainer * cfgcont = NULL);
 		static void run_command(const std::string& cmd, const std::string& param); // used for notifications only
 		static std::string run_program(char * argv[], const std::string& input);
 
@@ -56,6 +57,24 @@ class utils {
 
 		static std::vector<std::pair<unsigned int, unsigned int> > partition_indexes(unsigned int start, unsigned int end, unsigned int parts);
 
+		static std::string join(const std::vector<std::string>& strings, const std::string& separator);
+
+		static std::string censor_url(const std::string& url);
+
+		static std::string quote_for_stfl(std::string str);
+
+		static void trim_end(std::string& str);
+
+		static void trim(std::string& str);
+
+		static std::string quote(const std::string& str);
+
+		static unsigned int get_random_value(unsigned int max);
+
+		static std::string quote_if_necessary(const std::string& str);
+
+		static void set_common_curl_options(CURL * handle, configcontainer * cfg);
+
 	private:
 		static void append_escapes(std::string& str, char c);
 
