@@ -1,10 +1,13 @@
 #include <config.h>
 #include <configcontainer.h>
 #include <configparser.h>
+#include <exceptions.h>
 #include <logger.h>
 #include <sstream>
 #include <iostream>
+#include <algorithm>
 #include <utils.h>
+#include <cassert>
 
 #include <sys/types.h>
 #include <pwd.h>
@@ -37,6 +40,7 @@ configcontainer::configcontainer()
 	config_data["confirm-exit"]    = configdata("no", configdata::BOOL);
 	config_data["error-log"]       = configdata("", configdata::PATH);
 	config_data["notify-screen"]   = configdata("no", configdata::BOOL);
+	config_data["notify-always"]   = configdata("no", configdata::BOOL);
 	config_data["notify-xterm"]    = configdata("no", configdata::BOOL);
 	config_data["notify-program"]  = configdata("", configdata::PATH);
 	config_data["notify-format"]   = configdata(_("newsbeuter: finished reload, %f unread feeds (%n unread articles total)"), configdata::STR);
@@ -45,27 +49,32 @@ configcontainer::configcontainer()
 	config_data["bloglines-auth"]  = configdata("", configdata::STR);
 	config_data["bloglines-mark-read"] = configdata("no", configdata::BOOL);
 	config_data["bookmark-cmd"]    = configdata("", configdata::STR);
-	config_data["opml-url"]        = configdata("", configdata::STR);
+	config_data["opml-url"]        = configdata("", configdata::STR, true);
 	config_data["html-renderer"]   = configdata("internal", configdata::PATH);
 	config_data["feedlist-format"] = configdata("%4i %n %11u %t", configdata::STR);
 	config_data["articlelist-format"] = configdata("%4i %f %D   %?T?|%-17T|  &?%t", configdata::STR);
 	config_data["text-width"]      = configdata("0", configdata::INT);
 	config_data["always-display-description"] = configdata("false", configdata::BOOL);
 	config_data["reload-only-visible-feeds"] = configdata("false", configdata::BOOL);
-	config_data["article-sort-order"] = configdata("date", configdata::STR);
+	config_data["article-sort-order"] = configdata("date-asc", configdata::STR);
 	config_data["show-read-articles"] = configdata("yes", configdata::BOOL);
 	config_data["goto-next-feed"] = configdata("yes", configdata::BOOL);
 	config_data["display-article-progress"] = configdata("yes", configdata::BOOL);
 	config_data["show-keymap-hint"] = configdata("yes", configdata::BOOL);
 	config_data["download-timeout"] = configdata("30", configdata::INT);
 	config_data["download-retries"] = configdata("1", configdata::INT);
-	config_data["feed-sort-order"] = configdata("none", configdata::STR);
+	config_data["feed-sort-order"] = configdata("none-desc", configdata::STR);
 	config_data["reload-threads"] = configdata("1", configdata::INT);
+	config_data["keep-articles-days"] = configdata("0", configdata::INT);
+	config_data["bookmark-interactive"] = configdata("false", configdata::BOOL);
+	config_data["mark-as-read-on-hover"] = configdata("false", configdata::BOOL);
+	config_data["search-highlight-colors"] = configdata("black yellow bold", configdata::STR, true);
+	config_data["pager"] = configdata("internal", configdata::PATH);
+	config_data["history-limit"] = configdata("100", configdata::INT);
+	config_data["prepopulate-query-feeds"] = configdata("false", configdata::BOOL);
 
-
-	/* undocumented: */
+	/* title formats: */
 	config_data["feedlist-title-format"] = configdata(_("%N %V - Your feeds (%u unread, %t total)%?T? - tag `%T'&?"), configdata::STR);
-	config_data["urlview-title-format"] = configdata(_("%N %V - URLs"), configdata::STR);
 	config_data["articlelist-title-format"] = configdata(_("%N %V - Articles in feed '%T' (%u unread, %t total) - %U"), configdata::STR);
 	config_data["searchresult-title-format"] = configdata(_("%N %V - Search result (%u unread, %t total)"), configdata::STR);
 	config_data["filebrowser-title-format"] = configdata(_("%N %V - %?O?Open File&Save File? - %f"), configdata::STR);
@@ -74,6 +83,7 @@ configcontainer::configcontainer()
 	config_data["selectfilter-title-format"] = configdata(_("%N %V - Select Filter"), configdata::STR);
 	config_data["itemview-title-format"] = configdata(_("%N %V - Article '%T'"), configdata::STR);
 	config_data["urlview-title-format"] = configdata(_("%N %V - URLs"), configdata::STR);
+	config_data["dialogs-title-format"] = configdata(_("%N %V - Dialogs"), configdata::STR);
 }
 
 configcontainer::~configcontainer()
@@ -89,46 +99,46 @@ void configcontainer::register_commands(configparser& cfgparser)
 	}
 }
 
-action_handler_status configcontainer::handle_action(const std::string& action, const std::vector<std::string>& params) {
+void configcontainer::handle_action(const std::string& action, const std::vector<std::string>& params) {
 	configdata& cfgdata = config_data[action];
 
 	// configdata::INVALID indicates that the action didn't exist, and that the returned object was created ad-hoc.
 	if (cfgdata.type == configdata::INVALID) {
-		GetLogger().log(LOG_WARN, "configcontainer::handler_action: unknown action %s", action.c_str());
-		return AHS_INVALID_COMMAND;	
+		LOG(LOG_WARN, "configcontainer::handler_action: unknown action %s", action.c_str());
+		throw confighandlerexception(AHS_INVALID_COMMAND);
 	}
 
-	GetLogger().log(LOG_DEBUG, "configcontainer::handle_action: action = %s, type = %u", action.c_str(), cfgdata.type);
+	LOG(LOG_DEBUG, "configcontainer::handle_action: action = %s, type = %u", action.c_str(), cfgdata.type);
 
 	if (params.size() < 1) {
-		return AHS_TOO_FEW_PARAMS;
+		throw confighandlerexception(AHS_TOO_FEW_PARAMS);
 	}
 
 	switch (cfgdata.type) {
 		case configdata::BOOL:
 			if (!is_bool(params[0]))
-				return AHS_INVALID_PARAMS;
+				throw confighandlerexception(utils::strprintf(_("expected boolean value, found `%s' instead"), params[0].c_str()));
 			cfgdata.value = params[0];
-			return AHS_OK; 
+			break;
 
 		case configdata::INT:
 			if (!is_int(params[0]))
-				return AHS_INVALID_PARAMS;
+				throw confighandlerexception(utils::strprintf(_("expected integer value, found `%s' instead"), params[0].c_str()));
 			cfgdata.value = params[0];
-			return AHS_OK;	
+			break;
 
 		case configdata::STR:
 		case configdata::PATH:
-			cfgdata.value = params[0];
-			return AHS_OK;	
+			if (cfgdata.multi_option)
+				cfgdata.value = utils::join(params, " ");
+			else
+				cfgdata.value = params[0];
+			break;
 
 		default:
 			// should not happen
-			return AHS_INVALID_COMMAND;	
+			throw confighandlerexception(AHS_INVALID_COMMAND);
 	}
-
-	// should not happen
-	return AHS_INVALID_COMMAND;	
 }
 
 bool configcontainer::is_bool(const std::string& s) {
@@ -172,8 +182,61 @@ bool configcontainer::get_configvalue_as_bool(const std::string& key) {
 }
 
 void configcontainer::set_configvalue(const std::string& key, const std::string& value) {
-	GetLogger().log(LOG_DEBUG,"configcontainer::set_configvalue(%s,%s) called", key.c_str(), value.c_str());
+	LOG(LOG_DEBUG,"configcontainer::set_configvalue(%s,%s) called", key.c_str(), value.c_str());
 	config_data[key].value = value;
 }
 
+void configcontainer::reset_to_default(const std::string& key) {
+	config_data[key].value = config_data[key].default_value;
+}
+
+void configcontainer::toggle(const std::string& key) {
+	if (config_data[key].type == configdata::BOOL) {
+		set_configvalue(key, std::string(get_configvalue_as_bool(key) ? "false" : "true"));
+	}
+}
+
+void configcontainer::dump_config(std::vector<std::string>& config_output) {
+	for (std::map<std::string, configdata>::iterator it=config_data.begin();it!=config_data.end();it++) {
+		std::string configline = it->first + " ";
+		assert(it->second.type != configdata::INVALID);
+		switch (it->second.type) {
+		case configdata::BOOL:
+		case configdata::INT:
+			configline.append(it->second.value);
+			if (it->second.value != it->second.default_value)
+				configline.append(utils::strprintf(" # default: %s", it->second.default_value.c_str()));
+			break;
+		case configdata::STR:
+		case configdata::PATH:
+			if (it->second.multi_option) {
+				std::vector<std::string> tokens = utils::tokenize(it->second.value, " ");
+				for (std::vector<std::string>::iterator it=tokens.begin();it!=tokens.end();it++) {
+					configline.append(utils::quote(*it) + " ");
+				}
+			} else {
+				configline.append(utils::quote(it->second.value));
+				if (it->second.value != it->second.default_value) {
+					configline.append(utils::strprintf(" # default: %s", it->second.default_value.c_str()));
+				}
+			}
+			break;
+		case configdata::INVALID:
+			assert(0);
+			break;
+		}
+		config_output.push_back(configline);
+	}
+}
+
+std::vector<std::string> configcontainer::get_suggestions(const std::string& fragment) {
+	std::vector<std::string> result;
+	for (std::map<std::string, configdata>::iterator it=config_data.begin();it!=config_data.end();it++) {
+		if (it->first.substr(0, fragment.length()) == fragment)
+			result.push_back(it->first);
+	}
+	std::sort(result.begin(), result.end());
+	return result;
+}
+
 }
