@@ -12,6 +12,9 @@
 
 #include <sstream>
 #include <string>
+#include <algorithm>
+
+#include <cstring>
 
 #include <langinfo.h>
 
@@ -24,6 +27,10 @@ feedlist_formaction::feedlist_formaction(view * vv, std::string formstr)
 		auto_open(false), quit(false), apply_filter(false), search_dummy_feed(new rss_feed(v->get_ctrl()->get_cache())),
 		filterpos(0), set_filterpos(false), rxman(0), old_width(0) {
 	assert(true==m.parse(FILTER_UNREAD_FEEDS));
+	valid_cmds.push_back("tag");
+	valid_cmds.push_back("goto");
+	std::sort(valid_cmds.begin(), valid_cmds.end());
+	old_sort_order = v->get_cfg()->get_configvalue("feed-sort-order");
 }
 
 void feedlist_formaction::init() {
@@ -42,12 +49,9 @@ void feedlist_formaction::init() {
 	 * The feedlist_formaction is responsible for starting up the reloadthread, which is responsible
 	 * for regularly spawning downloadthreads.
 	 */
-	time_t reload_cycle = 60 * v->get_cfg()->get_configvalue_as_int("reload-time");
-	if (v->get_cfg()->get_configvalue_as_bool("auto-reload") == true) {
-		f->run(-1); // FRUN
-		reloadthread  * rt = new reloadthread(v->get_ctrl(), reload_cycle, v->get_cfg());
-		rt->start();
-	}
+	f->run(-1); // FRUN
+	reloadthread  * rt = new reloadthread(v->get_ctrl(), v->get_cfg());
+	rt->start();
 
 	apply_filter = !(v->get_cfg()->get_configvalue_as_bool("show-read-feeds"));
 }
@@ -60,14 +64,21 @@ void feedlist_formaction::prepare() {
 	if (old_width != width) {
 		do_redraw = true;
 		old_width = width;
-		GetLogger().log(LOG_DEBUG, "feedlist_formaction::prepare: apparent resize");
+		LOG(LOG_DEBUG, "feedlist_formaction::prepare: apparent resize");
+	}
+
+	std::string sort_order = v->get_cfg()->get_configvalue("feed-sort-order");
+	if (sort_order != old_sort_order) {
+		v->get_ctrl()->sort_feeds();
+		old_sort_order = sort_order;
+		do_redraw = true;
 	}
 
 	if (do_redraw) {
-		GetLogger().log(LOG_DEBUG, "feedlist_formaction::prepare: doing redraw");
-		do_redraw = false;
+		LOG(LOG_DEBUG, "feedlist_formaction::prepare: doing redraw");
 		v->get_ctrl()->update_feedlist();
 		set_pos();
+		do_redraw = false;
 	}
 }
 
@@ -83,7 +94,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 						std::istringstream x((*args)[0]);
 						x >> pos;
 					}
-					GetLogger().log(LOG_INFO, "feedlist_formaction: opening feed at position `%s'",feedpos.c_str());
+					LOG(LOG_INFO, "feedlist_formaction: opening feed at position `%s'",feedpos.c_str());
 					if (feeds_shown > 0 && feedpos.length() > 0) {
 						v->push_itemlist(pos);
 					} else {
@@ -93,7 +104,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_RELOAD: {
-				GetLogger().log(LOG_INFO, "feedlist_formaction: reloading feed at position `%s'",feedpos.c_str());
+				LOG(LOG_INFO, "feedlist_formaction: reloading feed at position `%s'",feedpos.c_str());
 				if (feeds_shown > 0 && feedpos.length() > 0) {
 					v->get_ctrl()->reload(pos);
 				} else {
@@ -107,6 +118,40 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 		case OP_RELOADURLS:
 			v->get_ctrl()->reload_urls_file();
 			break;
+		case OP_SORT: {
+				char c = v->confirm(_("Sort by (f)irsttag/(t)itle/(a)rticlecount/(u)nreadarticlecount/(n)one?"), _("ftaun"));
+				if (!c) break;
+				std::string result(1, c);
+				if (result == _("f")) {
+					v->get_cfg()->set_configvalue("feed-sort-order", "firsttag-desc");
+				} else if (result == _("t")) {
+					v->get_cfg()->set_configvalue("feed-sort-order", "title-desc");
+				} else if (result == _("a")) {
+					v->get_cfg()->set_configvalue("feed-sort-order", "articlecount-desc");
+				} else if (result == _("u")) {
+					v->get_cfg()->set_configvalue("feed-sort-order", "unreadarticlecount-desc");
+				} else if (result == _("n")) {
+					v->get_cfg()->set_configvalue("feed-sort-order", "none-desc");
+				}
+			}
+			break;
+		case OP_REVSORT: {
+				char c = v->confirm(_("Reverse Sort by (f)irsttag/(t)itle/(a)rticlecount/(u)nreadarticlecount/(n)one?"), _("ftaun"));
+				if (!c) break;
+				std::string result(1, c);
+				if (result == _("f")) {
+					v->get_cfg()->set_configvalue("feed-sort-order", "firsttag-asc");
+				} else if (result == _("t")) {
+					v->get_cfg()->set_configvalue("feed-sort-order", "title-asc");
+				} else if (result == _("a")) {
+					v->get_cfg()->set_configvalue("feed-sort-order", "articlecount-asc");
+				} else if (result == _("u")) {
+					v->get_cfg()->set_configvalue("feed-sort-order", "unreadarticlecount-asc");
+				} else if (result == _("n")) {
+					v->get_cfg()->set_configvalue("feed-sort-order", "none-asc");
+				}
+			}
+			break;
 		case OP_OPENINBROWSER: {
 				std::tr1::shared_ptr<rss_feed> feed = v->get_ctrl()->get_feed(pos);
 				if (feed) {
@@ -115,7 +160,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_RELOADALL:
-			GetLogger().log(LOG_INFO, "feedlist_formaction: reloading all feeds");
+			LOG(LOG_INFO, "feedlist_formaction: reloading all feeds");
 			{
 				bool reload_only_visible_feeds = v->get_cfg()->get_configvalue_as_bool("reload-only-visible-feeds");
 				std::vector<int> idxs;
@@ -126,7 +171,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_MARKFEEDREAD: {
-				GetLogger().log(LOG_INFO, "feedlist_formaction: marking feed read at position `%s'",feedpos.c_str());
+				LOG(LOG_INFO, "feedlist_formaction: marking feed read at position `%s'",feedpos.c_str());
 				if (feeds_shown > 0 && feedpos.length() > 0) {
 					v->set_status(_("Marking feed read..."));
 					try {
@@ -143,7 +188,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			break;
 		case OP_TOGGLESHOWREAD:
 			m.parse(FILTER_UNREAD_FEEDS);
-			GetLogger().log(LOG_INFO, "feedlist_formaction: toggling show-read-feeds");
+			LOG(LOG_INFO, "feedlist_formaction: toggling show-read-feeds");
 			if (v->get_cfg()->get_configvalue_as_bool("show-read-feeds")) {
 				v->get_cfg()->set_configvalue("show-read-feeds","no");
 				apply_filter = true;
@@ -156,7 +201,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			break;
 		case OP_NEXTUNREAD: {
 				unsigned int local_tmp;
-				GetLogger().log(LOG_INFO, "feedlist_formaction: jumping to next unred feed");
+				LOG(LOG_INFO, "feedlist_formaction: jumping to next unred feed");
 				if (!jump_to_next_unread_feed(local_tmp)) {
 					v->show_error(_("No feeds with unread items."));
 				}
@@ -164,14 +209,22 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			break;
 		case OP_PREVUNREAD: {
 				unsigned int local_tmp;
-				GetLogger().log(LOG_INFO, "feedlist_formaction: jumping to previous unred feed");
+				LOG(LOG_INFO, "feedlist_formaction: jumping to previous unred feed");
 				if (!jump_to_previous_unread_feed(local_tmp)) {
 					v->show_error(_("No feeds with unread items."));
 				}
 			}
 			break;
+		case OP_RANDOMUNREAD: {
+				unsigned int local_tmp;
+				LOG(LOG_INFO, "feedlist_formaction: jumping to random unread feed");
+				if (!jump_to_random_unread_feed(local_tmp)) {
+					v->show_error(_("No feeds with unread items."));
+				}
+			}
+			break;
 		case OP_MARKALLFEEDSREAD:
-			GetLogger().log(LOG_INFO, "feedlist_formaction: marking all feeds read");
+			LOG(LOG_INFO, "feedlist_formaction: marking all feeds read");
 			v->set_status(_("Marking all feeds read..."));
 			v->get_ctrl()->catchup_all();
 			v->set_status("");
@@ -211,7 +264,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 					filterhistory.add_line(newfilter);
 					if (newfilter.length() > 0) {
 						if (!m.parse(newfilter)) {
-							v->show_error(_("Error: couldn't parse filter command!"));
+							v->show_error(utils::strprintf(_("Error: couldn't parse filter command `%s': %s"), newfilter.c_str(), m.get_parse_error().c_str()));
 							m.parse(FILTER_UNREAD_FEEDS);
 						} else {
 							save_filterpos();
@@ -259,7 +312,7 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			v->get_ctrl()->edit_urls_file();
 			break;
 		case OP_QUIT:
-			GetLogger().log(LOG_INFO, "feedlist_formaction: quitting");
+			LOG(LOG_INFO, "feedlist_formaction: quitting");
 			if (automatic || !v->get_cfg()->get_configvalue_as_bool("confirm-exit") || v->confirm(_("Do you really want to quit (y:Yes n:No)? "), _("yn")) == *_("y")) {
 				quit = true;
 			}
@@ -271,7 +324,8 @@ void feedlist_formaction::process_operation(operation op, bool automatic, std::v
 			break;
 	}
 	if (quit) {
-		v->pop_current_formaction();
+		while (v->formaction_stack_size() > 0)
+			v->pop_current_formaction();
 	}
 }
 
@@ -283,6 +337,7 @@ void feedlist_formaction::update_visible_feeds(std::vector<std::tr1::shared_ptr<
 	unsigned int i = 0;
 
 	for (std::vector<std::tr1::shared_ptr<rss_feed> >::iterator it = feeds.begin(); it != feeds.end(); ++it, ++i) {
+		(*it)->set_index(i+1);
 		if ((tag == "" || (*it)->matches_tag(tag)) && (!apply_filter || m.matches(it->get()))) {
 			visible_feeds.push_back(feedptr_pos_pair(*it,i));
 		}
@@ -297,7 +352,7 @@ void feedlist_formaction::set_feedlist(std::vector<std::tr1::shared_ptr<rss_feed
 	unsigned int width = utils::to_u(f->get("feeds:w"));
 
 	unsigned int i = 0;
-	unsigned int unread_feeds = 0;
+	unread_feeds = 0;
 
 	std::string feedlist_format = v->get_cfg()->get_configvalue("feedlist-format");
 
@@ -314,6 +369,8 @@ void feedlist_formaction::set_feedlist(std::vector<std::tr1::shared_ptr<rss_feed
 		listfmt.add_line(format_line(feedlist_format, it->first, it->second, width), it->second);
 	}
 
+	total_feeds = i;
+
 	f->modify("feeds","replace_inner",listfmt.format_list(rxman, "feedlist"));
 
 	std::string title_format = v->get_cfg()->get_configvalue("feedlist-title-format");
@@ -352,21 +409,21 @@ bool feedlist_formaction::jump_to_previous_unread_feed(unsigned int& feedpos) {
 	unsigned int curpos;
 	std::istringstream is(f->get("feedpos"));
 	is >> curpos;
-	GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: searching for unread feed");
+	LOG(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: searching for unread feed");
 
 	for (int i=curpos-1;i>=0;--i) {
-		GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
+		LOG(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
 		if (visible_feeds[i].first->unread_item_count() > 0) {
-			GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: hit");
+			LOG(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: hit");
 			f->set("feedpos", utils::to_s(i));
 			feedpos = visible_feeds[i].second;
 			return true;
 		}
 	}
 	for (int i=visible_feeds.size()-1;i>=static_cast<int>(curpos);--i) {
-		GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
+		LOG(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
 		if (visible_feeds[i].first->unread_item_count() > 0) {
-			GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: hit");
+			LOG(LOG_DEBUG, "feedlist_formaction::jump_to_previous_unread_feed: hit");
 			f->set("feedpos", utils::to_s(i));
 			feedpos = visible_feeds[i].second;
 			return true;
@@ -379,7 +436,7 @@ void feedlist_formaction::goto_feed(const std::string& str) {
 	unsigned int curpos;
 	std::istringstream is(f->get("feedpos"));
 	is >> curpos;
-	GetLogger().log(LOG_DEBUG, "feedlist_formaction::goto_feed: curpos = %u str = `%s'", curpos, str.c_str());
+	LOG(LOG_DEBUG, "feedlist_formaction::goto_feed: curpos = %u str = `%s'", curpos, str.c_str());
 	for (unsigned int i=curpos+1;i<visible_feeds.size();++i) {
 		if (strcasestr(visible_feeds[i].first->title().c_str(), str.c_str()) != NULL) {
 			f->set("feedpos", utils::to_s(i));
@@ -394,25 +451,46 @@ void feedlist_formaction::goto_feed(const std::string& str) {
 	}
 }
 
+bool feedlist_formaction::jump_to_random_unread_feed(unsigned int& feedpos) {
+	bool unread_feeds_available = false;
+	for (unsigned int i=0;i<visible_feeds.size();++i) {
+		if (visible_feeds[i].first->unread_item_count() > 0) {
+			unread_feeds_available = true;
+			break;
+		}
+	}
+	if (unread_feeds_available) {
+		for (;;) {
+			unsigned int pos = utils::get_random_value(visible_feeds.size());
+			if (visible_feeds[pos].first->unread_item_count() > 0) {
+				f->set("feedpos", utils::to_s(pos));
+				feedpos = visible_feeds[pos].second;
+				break;
+			}
+		}
+	}
+	return unread_feeds_available;
+}
+
 bool feedlist_formaction::jump_to_next_unread_feed(unsigned int& feedpos) {
 	unsigned int curpos;
 	std::istringstream is(f->get("feedpos"));
 	is >> curpos;
-	GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: searching for unread feed");
+	LOG(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: searching for unread feed");
 
 	for (unsigned int i=curpos+1;i<visible_feeds.size();++i) {
-		GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
+		LOG(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
 		if (visible_feeds[i].first->unread_item_count() > 0) {
-			GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: hit");
+			LOG(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: hit");
 			f->set("feedpos", utils::to_s(i));
 			feedpos = visible_feeds[i].second;
 			return true;
 		}
 	}
 	for (unsigned int i=0;i<=curpos;++i) {
-		GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
+		LOG(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: visible_feeds[%u] unread items: %u", i, visible_feeds[i].first->unread_item_count());
 		if (visible_feeds[i].first->unread_item_count() > 0) {
-			GetLogger().log(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: hit");
+			LOG(LOG_DEBUG, "feedlist_formaction::jump_to_next_unread_feed: hit");
 			f->set("feedpos", utils::to_s(i));
 			feedpos = visible_feeds[i].second;
 			return true;
@@ -489,9 +567,18 @@ void feedlist_formaction::mark_pos_if_visible(unsigned int pos) {
 	v->get_ctrl()->update_visible_feeds();
 	for (std::vector<feedptr_pos_pair>::iterator it=visible_feeds.begin();it!=visible_feeds.end();++it, ++vpos) {
 		if (it->second == pos) {
-			GetLogger().log(LOG_DEBUG, "feedlist_formaction::mark_pos_if_visible: match, setting position to %u", vpos);
+			LOG(LOG_DEBUG, "feedlist_formaction::mark_pos_if_visible: match, setting position to %u", vpos);
 			f->set("feedpos", utils::to_s(vpos));
-			break;
+			return;
+		}
+	}
+	vpos = 0;
+	pos = v->get_ctrl()->get_pos_of_next_unread(pos);
+	for (std::vector<feedptr_pos_pair>::iterator it=visible_feeds.begin();it!=visible_feeds.end();++it, ++vpos) {
+		if (it->second == pos) {
+			LOG(LOG_DEBUG, "feedlist_formaction::mark_pos_if_visible: match in 2nd try, setting position to %u", vpos);
+			f->set("feedpos", utils::to_s(vpos));
+			return;
 		}
 	}
 }
@@ -538,7 +625,7 @@ void feedlist_formaction::op_end_setfilter() {
 
 void feedlist_formaction::op_start_search() {
 	std::string searchphrase = qna_responses[0];
-	GetLogger().log(LOG_DEBUG, "feedlist_formaction::op_start_search: starting search for `%s'", searchphrase.c_str());
+	LOG(LOG_DEBUG, "feedlist_formaction::op_start_search: starting search for `%s'", searchphrase.c_str());
 	if (searchphrase.length() > 0) {
 		v->set_status(_("Searching..."));
 		searchhistory.add_line(searchphrase);
@@ -551,8 +638,10 @@ void feedlist_formaction::op_start_search() {
 			return;
 		}
 		if (items.size() > 0) {
+			search_dummy_feed->item_mutex.lock();
 			search_dummy_feed->items() = items;
-			v->push_searchresult(search_dummy_feed);
+			search_dummy_feed->item_mutex.unlock();
+			v->push_searchresult(search_dummy_feed, searchphrase);
 		} else {
 			v->show_error(_("No results."));
 		}
@@ -592,7 +681,7 @@ void feedlist_formaction::set_pos() {
 std::string feedlist_formaction::get_title(std::tr1::shared_ptr<rss_feed> feed) {
 	std::string title = feed->title();
 	if (title.length()==0)
-		title = feed->rssurl();
+		title = utils::censor_url(feed->rssurl());
 	if (title.length()==0)
 		title = "<no title>";
 	return title;
@@ -607,14 +696,15 @@ std::string feedlist_formaction::format_line(const std::string& feedlist_format,
 	fmt.register_fmt('n', unread_count > 0 ? "N" : " ");
 	fmt.register_fmt('t', get_title(feed));
 	fmt.register_fmt('T', feed->get_firsttag());
-	fmt.register_fmt('l', feed->link());
-	fmt.register_fmt('L', feed->rssurl());
+	fmt.register_fmt('l', utils::censor_url(feed->link()));
+	fmt.register_fmt('L', utils::censor_url(feed->rssurl()));
 	fmt.register_fmt('d', feed->description());
 
-	std::string format = fmt.do_format(feedlist_format, width);
-	GetLogger().log(LOG_DEBUG, "feedlist_formaction::set_feedlist: format result = %s", format.c_str());
+	return fmt.do_format(feedlist_format, width);
+}
 
-	return format;
+std::string feedlist_formaction::title() {
+	return utils::strprintf(_("Feed List - %u unread, %u total"), unread_feeds, total_feeds);
 }
 
 }
