@@ -4,14 +4,23 @@
 #include <config.h>
 #include <logger.h>
 #include <cassert>
+#include <exceptions.h>
 
 namespace newsbeuter {
 
 history formaction::searchhistory;
 history formaction::cmdlinehistory;
 
-formaction::formaction(view * vv, std::string formstr) : v(vv), f(0), do_redraw(true) { 
-	f = new stfl::form(formstr);
+formaction::formaction(view * vv, std::string formstr) : v(vv), f(new stfl::form(formstr)), do_redraw(true) { 
+	if (v) {
+		if (v->get_cfg()->get_configvalue_as_bool("show-keymap-hint") == false) {
+			f->set("showhint", "0");
+		}
+	}
+	valid_cmds.push_back("set");
+	valid_cmds.push_back("quit");
+	valid_cmds.push_back("source");
+	valid_cmds.push_back("dumpconfig");
 }
 
 void formaction::set_keymap_hints() {
@@ -22,11 +31,9 @@ void formaction::recalculate_form() {
 	f->run(-3);
 }
 
-formaction::~formaction() { 
-	delete f;
-}
+formaction::~formaction() { }
 
-stfl::form * formaction::get_form() {
+std::tr1::shared_ptr<stfl::form> formaction::get_form() {
 	return f;
 }
 
@@ -55,12 +62,17 @@ std::string formaction::get_value(const std::string& value) {
 void formaction::start_cmdline() {
 	std::vector<qna_pair> qna;
 	qna.push_back(qna_pair(":", ""));
+	v->inside_cmdline(true);
 	this->start_qna(qna, OP_INT_END_CMDLINE, &formaction::cmdlinehistory);
 }
 
 
 void formaction::process_op(operation op, bool automatic, std::vector<std::string> * args) {
 	switch (op) {
+		case OP_REDRAW:
+			LOG(LOG_DEBUG, "formaction::process_op: redrawing screen");
+			stfl::reset();
+			break;
 		case OP_CMDLINE: 
 			start_cmdline();
 			break;
@@ -72,23 +84,29 @@ void formaction::process_op(operation op, bool automatic, std::vector<std::strin
 						cmdline.append(utils::strprintf("%s ", stfl::quote(*it).c_str()));
 					}
 				}
-				GetLogger().log(LOG_DEBUG, "formaction::process_op: running commandline `%s'", cmdline.c_str());
+				LOG(LOG_DEBUG, "formaction::process_op: running commandline `%s'", cmdline.c_str());
 				this->handle_cmdline(cmdline);
 			} else {
-				GetLogger().log(LOG_WARN, "formaction::process_op: got OP_INT_SET, but not automatic");
+				LOG(LOG_WARN, "formaction::process_op: got OP_INT_SET, but not automatic");
 			}
 			break;
 		case OP_INT_CANCEL_QNA:
 			f->modify("lastline","replace","{hbox[lastline] .expand:0 {label[msglabel] .expand:h text[msg]:\"\"}}");
+			v->inside_qna(false);
+			v->inside_cmdline(false);
 			break;
 		case OP_INT_QNA_NEXTHIST:
 			if (qna_history) {
-				f->set("qna_value", qna_history->next());
+				std::string entry = qna_history->next();
+				f->set("qna_value", entry);
+				f->set("qna_value_pos", utils::to_s(entry.length()));
 			}
 			break;
 		case OP_INT_QNA_PREVHIST:
 			if (qna_history) {
-				f->set("qna_value", qna_history->prev());
+				std::string entry = qna_history->prev();
+				f->set("qna_value", entry);
+				f->set("qna_value_pos", utils::to_s(entry.length()));
 			}
 			break;
 		case OP_INT_END_QUESTION:
@@ -98,11 +116,54 @@ void formaction::process_op(operation op, bool automatic, std::vector<std::strin
 			qna_responses.push_back(f->get("qna_value"));
 			start_next_question();
 			break;
+		case OP_VIEWDIALOGS:
+			v->view_dialogs();
+			break;
+		case OP_NEXTDIALOG:
+			v->goto_next_dialog();
+			break;
+		case OP_PREVDIALOG:
+			v->goto_prev_dialog();
+			break;
 		default:
 			this->process_operation(op, automatic, args);
 	}
 }
 
+std::vector<std::string> formaction::get_suggestions(const std::string& fragment) {
+	LOG(LOG_DEBUG, "formaction::get_suggestions: fragment = %s", fragment.c_str());
+	std::vector<std::string> result;
+	// first check all formaction command suggestions
+	for (std::vector<std::string>::iterator it=valid_cmds.begin();it!=valid_cmds.end();it++) {
+		LOG(LOG_DEBUG, "formaction::get_suggestions: extracted part: %s", it->substr(0, fragment.length()).c_str());
+		if (it->substr(0, fragment.length()) == fragment) {
+			LOG(LOG_DEBUG, "...and it matches.");
+			result.push_back(*it);
+		}
+	}
+	if (result.size()==0) {
+		std::vector<std::string> tokens = utils::tokenize_quoted(fragment, " \t=");
+		if (tokens.size() >= 1) {
+			if (tokens[0] == "set") {
+				if (tokens.size() < 3) {
+					std::vector<std::string> variable_suggestions;
+					std::string variable_fragment;
+					if (tokens.size() > 1)
+						variable_fragment = tokens[1];
+					variable_suggestions = v->get_cfg()->get_suggestions(variable_fragment);
+					for (std::vector<std::string>::iterator it=variable_suggestions.begin();it!=variable_suggestions.end();it++) {
+						std::string line = fragment + it->substr(variable_fragment.length(), it->length()-variable_fragment.length());
+						result.push_back(line);
+						LOG(LOG_DEBUG, "formaction::get_suggestions: suggested %s", line.c_str());
+					}
+				}
+			}
+		}
+	}
+	LOG(LOG_DEBUG, "formaction::get_suggestions: %u suggestions", result.size());
+	return result;
+}
+
 void formaction::handle_cmdline(const std::string& cmdline) {
 	/*
 	 * this is the command line handling that is available on all dialogs.
@@ -123,9 +184,23 @@ void formaction::handle_cmdline(const std::string& cmdline) {
 			if (tokens.size()==0) {
 				v->show_error(_("usage: set <variable>[=<value>]"));
 			} else if (tokens.size()==1) {
-				v->set_status(utils::strprintf("  %s=%s", tokens[0].c_str(), cfg->get_configvalue(tokens[0]).c_str()));
+				std::string var = tokens[0];
+				if (var.length() > 0) {
+					if (var[var.length()-1] == '!') {
+						var.erase(var.length()-1);
+						cfg->toggle(var);
+						set_redraw(true);
+					} else if (var[var.length()-1] == '&') {
+						var.erase(var.length()-1);
+						cfg->reset_to_default(var);
+						set_redraw(true);
+					}
+					v->set_status(utils::strprintf("  %s=%s", var.c_str(), utils::quote_if_necessary(cfg->get_configvalue(var)).c_str()));
+				}
 			} else if (tokens.size()==2) {
-				cfg->set_configvalue(tokens[0], tokens[1]);
+				std::string result = configparser::evaluate_backticks(tokens[1]);
+				utils::trim_end(result);
+				cfg->set_configvalue(tokens[0], result);
 				set_redraw(true); // because some configuration value might have changed something UI-related
 			} else {
 				v->show_error(_("usage: set <variable>[=<value>]"));
@@ -134,6 +209,28 @@ void formaction::handle_cmdline(const std::string& cmdline) {
 			while (v->formaction_stack_size() > 0) {
 				v->pop_current_formaction();
 			}
+		} else if (cmd == "source") {
+			if (tokens.size()==0) {
+				v->show_error(_("usage: source <file> [...]"));
+			} else {
+				for (std::vector<std::string>::iterator it=tokens.begin();it!=tokens.end();it++) {
+					try {
+						v->get_ctrl()->load_configfile(utils::resolve_tilde(*it));
+					} catch (const configexception& ex) {
+						v->show_error(ex.what());
+						break;
+					}
+				}
+			}
+		} else if (cmd == "dumpconfig") {
+			if (tokens.size()!=1) {
+				v->show_error(_("usage: dumpconfig <file>"));
+			} else {
+				v->get_ctrl()->dump_config(utils::resolve_tilde(tokens[1]));
+				v->show_error(utils::strprintf(_("Saved configuration to %s"), tokens[1].c_str()));
+			}
+		} else {
+			v->show_error(utils::strprintf(_("Not a command: %s"), cmdline.c_str()));
 		}
 	}
 }
@@ -153,10 +250,13 @@ void formaction::start_qna(const std::vector<qna_pair>& prompts, operation finis
 	qna_responses.clear();
 	finish_operation = finish_op;
 	qna_history = h;
+	v->inside_qna(true);
 	start_next_question();
 }
 
 void formaction::finished_qna(operation op) {
+	v->inside_qna(false);
+	v->inside_cmdline(false);
 	switch (op) {
 		/*
 		 * since bookmarking is available in several formactions, I decided to put this into
@@ -174,7 +274,8 @@ void formaction::finished_qna(operation op) {
 				if (retval.length() == 0) {
 					v->set_status(_("Saved bookmark."));
 				} else {
-					v->set_status((std::string(_("Error while saving bookmark: ")) + retval).c_str());
+					v->set_status(std::string(_("Error while saving bookmark: ")) + retval);
+					LOG(LOG_DEBUG, "formaction::finished_qna: error while saving bookmark, retval = `%s'", retval.c_str());
 				}
 			}
 			break;
@@ -182,7 +283,7 @@ void formaction::finished_qna(operation op) {
 				f->set_focus("feeds");
 				std::string cmdline = qna_responses[0];
 				formaction::cmdlinehistory.add_line(cmdline);
-				GetLogger().log(LOG_DEBUG,"formaction: commandline = `%s'", cmdline.c_str());
+				LOG(LOG_DEBUG,"formaction: commandline = `%s'", cmdline.c_str());
 				this->handle_cmdline(cmdline);
 			}
 			break;
@@ -193,7 +294,7 @@ void formaction::finished_qna(operation op) {
 
 
 void formaction::start_bookmark_qna(const std::string& default_title, const std::string& default_url, const std::string& default_desc) {
-	GetLogger().log(LOG_DEBUG, "formaction::start_bookmark_qna: OK, starting bookmark Q&A...");
+	LOG(LOG_DEBUG, "formaction::start_bookmark_qna: starting bookmark Q&A... default_title = %s default_url = %s default_desc = %s", default_title.c_str(), default_url.c_str(), default_desc.c_str());
 	std::vector<qna_pair> prompts;
 
 	prompts.push_back(qna_pair(_("URL: "), default_url));
@@ -212,6 +313,8 @@ void formaction::start_next_question() {
 		replacestr.append(stfl::quote(qna_prompts[0].first));
 		replacestr.append("}{input[qnainput] on_ESC:cancel-qna on_UP:qna-prev-history on_DOWN:qna-next-history on_ENTER:end-question modal:1 .expand:h text[qna_value]:");
 		replacestr.append(stfl::quote(qna_prompts[0].second));
+		replacestr.append(" pos[qna_value_pos]:");
+		replacestr.append(utils::to_s(qna_prompts[0].second.length()));
 		replacestr.append("}}");
 		qna_prompts.erase(qna_prompts.begin());
 		f->modify("lastline", "replace", replacestr);
@@ -225,5 +328,15 @@ void formaction::start_next_question() {
 	}
 }
 
+void formaction::load_histories(const std::string& searchfile, const std::string& cmdlinefile) {
+	searchhistory.load_from_file(searchfile);
+	cmdlinehistory.load_from_file(cmdlinefile);
+}
+
+void formaction::save_histories(const std::string& searchfile, const std::string& cmdlinefile, unsigned int limit) {
+	searchhistory.save_to_file(searchfile, limit);
+	cmdlinehistory.save_to_file(cmdlinefile, limit);
+}
+
 
 }
