@@ -11,9 +11,9 @@
 #include <libgen.h>
 #include <config.h>
 
-using namespace newsbeuter;
+namespace newsbeuter {
 
-htmlrenderer::htmlrenderer(unsigned int width) : w(width) { 
+htmlrenderer::htmlrenderer(unsigned int width, bool raw) : w(width), raw_(raw) { 
 	tags["a"] = TAG_A;
 	tags["embed"] = TAG_EMBED;
 	tags["br"] = TAG_BR;
@@ -31,6 +31,9 @@ htmlrenderer::htmlrenderer(unsigned int width) : w(width) {
 	tags["sup"] = TAG_SUP;
 	tags["sub"] = TAG_SUB;
 	tags["hr"] = TAG_HR;
+	tags["b"] = TAG_STRONG;
+	tags["strong"] = TAG_STRONG;
+	tags["u"] = TAG_UNDERLINE;
 }
 
 void htmlrenderer::render(const std::string& source, std::vector<std::string>& lines, std::vector<linkpair>& links, const std::string& url) {
@@ -83,29 +86,37 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 				std::transform(tagname.begin(), tagname.end(), tagname.begin(), ::tolower);
 				current_tag = tags[tagname];
 
-				GetLogger().log(LOG_DEBUG,"htmlrenderer::render: found start tag %s (id = %u)",tagname.c_str(), current_tag);
 				switch (current_tag) {
 					case TAG_A: {
 							std::string link;
 							try {
 								link = xpp.getAttributeValue("href");
 							} catch (const std::invalid_argument& ) {
-								GetLogger().log(LOG_WARN,"htmlrenderer::render: found a tag with no href attribute");
+								LOG(LOG_WARN,"htmlrenderer::render: found a tag with no href attribute");
 								link = "";
 							}
 							if (link.length() > 0) {
-								link_num = add_link(links,utils::absolute_url(url,link), LINK_HREF);
-								curline.append("[");
+								link_num = add_link(links,utils::censor_url(utils::absolute_url(url,link)), LINK_HREF);
+								if (!raw_) 
+									curline.append("<u>");
 							}
 						}
 						break;
+					case TAG_STRONG:
+						if (!raw_) 
+							curline.append("<b>");
+						break;
+					case TAG_UNDERLINE:
+						if (!raw_) 
+							curline.append("<u>");
+						break;
 
 					case TAG_EMBED: {
 							std::string type;
 							try {
 								type = xpp.getAttributeValue("type");
 							} catch (const std::invalid_argument& ) {
-								GetLogger().log(LOG_WARN, "htmlrenderer::render: found embed object without type attribute");
+								LOG(LOG_WARN, "htmlrenderer::render: found embed object without type attribute");
 								type = "";
 							}
 							if (type == "application/x-shockwave-flash") {
@@ -113,11 +124,11 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 								try {
 									link = xpp.getAttributeValue("src");
 								} catch (const std::invalid_argument& ) {
-									GetLogger().log(LOG_WARN, "htmlrenderer::render: found embed object without src attribute");
+									LOG(LOG_WARN, "htmlrenderer::render: found embed object without src attribute");
 									link = "";
 								}
 								if (link.length() > 0) {
-									unsigned int link_num = add_link(links,utils::absolute_url(url,link), LINK_EMBED);
+									unsigned int link_num = add_link(links,utils::censor_url(utils::absolute_url(url,link)), LINK_EMBED);
 									curline.append(utils::strprintf("[%s %u]", _("embedded flash:"), link_num));
 								}
 							}
@@ -125,7 +136,6 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 						break;
 
 					case TAG_BR:
-						GetLogger().log(LOG_DEBUG, "htmlrenderer::render: pushing back `%s'", curline.c_str());
 						lines.push_back(curline);
 						prepare_newline(curline, indent_level);	
 						break;
@@ -146,11 +156,11 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 							try {
 								imgurl = xpp.getAttributeValue("src");
 							} catch (const std::invalid_argument& ) {
-								GetLogger().log(LOG_WARN,"htmlrenderer::render: found img tag with no src attribute");
+								LOG(LOG_WARN,"htmlrenderer::render: found img tag with no src attribute");
 								imgurl = "";
 							}
 							if (imgurl.length() > 0) {
-								unsigned int link_num = add_link(links,utils::absolute_url(url,imgurl), LINK_IMG);
+								unsigned int link_num = add_link(links,utils::censor_url(utils::absolute_url(url,imgurl)), LINK_IMG);
 								curline.append(utils::strprintf("[%s %u]", _("image"), link_num));
 								image_count++;
 							}
@@ -195,6 +205,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 					case TAG_LI:
 						if (inside_li) {
 							indent_level-=2;
+							if (indent_level < 0) indent_level = 0;
 							if (line_is_nonempty(curline))
 								lines.push_back(curline);
 							prepare_newline(curline, indent_level);	
@@ -244,7 +255,8 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 						lines.push_back(std::string(" ") + std::string(w - 2, '-') + std::string(" "));
 						prepare_newline(curline, indent_level);
 						break;
-
+					default:
+						break;
 				}
 				break;
 
@@ -253,13 +265,10 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 				std::transform(tagname.begin(), tagname.end(), tagname.begin(), ::tolower);
 				current_tag = tags[tagname];
 
-				GetLogger().log(LOG_DEBUG, "htmlrenderer::render: found end tag %s (id = %u)",tagname.c_str(), current_tag);
-
 				switch (current_tag) {
 					case TAG_BLOCKQUOTE:
 						--indent_level;
-						if (indent_level < 0)
-							indent_level = 0;
+						if (indent_level < 0) indent_level = 0;
 						if (line_is_nonempty(curline))
 							lines.push_back(curline);
 						lines.push_back("");
@@ -271,6 +280,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 						inside_list = false;
 						if (inside_li) {
 							indent_level-=2;
+							if (indent_level < 0) indent_level = 0;
 							if (line_is_nonempty(curline))
 								lines.push_back(curline);
 							prepare_newline(curline, indent_level);
@@ -290,6 +300,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 
 					case TAG_DD:
 						indent_level-=4;
+						if (indent_level < 0) indent_level = 0;
 						if (line_is_nonempty(curline))
 							lines.push_back(curline);
 						lines.push_back("");
@@ -302,6 +313,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 
 					case TAG_LI:
 						indent_level-=2;
+						if (indent_level < 0) indent_level = 0;
 						inside_li = false;
 						if (line_is_nonempty(curline))
 							lines.push_back(curline);
@@ -331,11 +343,23 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 
 					case TAG_A:
 						if (link_num != -1) {
-							curline.append(utils::strprintf("][%d]", link_num));
+							if (!raw_)
+								curline.append("</>");
+							curline.append(utils::strprintf("[%d]", link_num));
 							link_num = -1;
 						}
 						break;
 
+					case TAG_UNDERLINE:
+						if (!raw_)
+							curline.append("</>");
+						break;
+
+					case TAG_STRONG:
+						if (!raw_)
+							curline.append("</>");
+						break;
+
 					case TAG_EMBED:
 					case TAG_BR:
 					case TAG_ITUNESHACK:
@@ -343,25 +367,24 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 					case TAG_HR:
 						// ignore closing tags
 						break;
+					default:
+						break;
 				}
 				break;
 
 			case tagsouppullparser::TEXT:
 				{
-					GetLogger().log(LOG_DEBUG,"htmlrenderer::render: found text `%s'",xpp.getText().c_str());
 					if (itunes_hack) {
-						std::vector<std::string> words = utils::tokenize_nl(xpp.getText());
+						std::vector<std::string> words = utils::tokenize_nl(utils::quote_for_stfl(xpp.getText()));
 						for (std::vector<std::string>::iterator it=words.begin();it!=words.end();++it) {
 							if (*it == "\n") {
 								lines.push_back(curline);
 								prepare_newline(curline, indent_level);
 							} else {
-								GetLogger().log(LOG_DEBUG, "htmlrenderer::render: tokenizing `%s'", it->c_str());
 								std::vector<std::string> words2 = utils::tokenize_spaced(*it);
 								unsigned int i=0;
 								bool new_line = false;
 								for (std::vector<std::string>::iterator it2=words2.begin();it2!=words2.end();++it2,++i) {
-									GetLogger().log(LOG_DEBUG, "htmlrenderer::render: token[%u] = `%s'", i, it2->c_str());
 									if ((curline.length() + it2->length()) >= w) {
 										if (line_is_nonempty(curline))
 											lines.push_back(curline);
@@ -379,7 +402,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 							}
 						}
 					} else if (inside_pre) {
-						std::vector<std::string> words = utils::tokenize_nl(xpp.getText());
+						std::vector<std::string> words = utils::tokenize_nl(utils::quote_for_stfl(xpp.getText()));
 						for (std::vector<std::string>::iterator it=words.begin();it!=words.end();++it) {
 							if (*it == "\n") {
 								lines.push_back(curline);
@@ -389,7 +412,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 							}
 						}
 					} else {
-						std::string s = xpp.getText();
+						std::string s = utils::quote_for_stfl(xpp.getText());
 						while (s.length() > 0 && s[0] == '\n')
 							s.erase(0, 1);
 						std::vector<std::string> words = utils::tokenize_spaced(s);
@@ -401,9 +424,7 @@ void htmlrenderer::render(std::istream& input, std::vector<std::string>& lines,
 							words.erase(words.begin());
 						}
 
-						GetLogger().log(LOG_DEBUG, "htmlrenderer::render: tokenized `%s'", xpp.getText().c_str());
 						for (std::vector<std::string>::iterator it=words.begin();it!=words.end();++it,++i) {
-							GetLogger().log(LOG_DEBUG, "htmlrenderer::render: token[%u] = `%s'", i, it->c_str());
 							if ((curline.length() + it->length()) >= w) {
 								if (line_is_nonempty(curline))
 									lines.push_back(curline);
@@ -449,9 +470,7 @@ std::string htmlrenderer::type2str(link_type type) {
 
 void htmlrenderer::prepare_newline(std::string& line, int indent_level) {
 	line = "";
-	for (int i=0;i<indent_level;++i) {
-		line.append("  ");	
-	}
+	line.append(indent_level*2, ' ');
 }
 
 bool htmlrenderer::line_is_nonempty(const std::string& line) {
@@ -461,3 +480,5 @@ bool htmlrenderer::line_is_nonempty(const std::string& line) {
 	}
 	return false;
 }
+
+}
