@@ -20,8 +20,9 @@ namespace newsbeuter {
 itemlist_formaction::itemlist_formaction(view * vv, std::string formstr)
 	: formaction(vv,formstr), apply_filter(false), update_visible_items(true), show_searchresult(false),
 		search_dummy_feed(new rss_feed(v->get_ctrl()->get_cache())),
-		set_filterpos(false), filterpos(0), rxman(0), old_width(0) {
+		set_filterpos(false), filterpos(0), rxman(0), old_width(0), old_itempos(-1) {
 	assert(true==m.parse(FILTER_UNREAD_ITEMS));
+	old_sort_order = v->get_cfg()->get_configvalue("article-sort-order");
 }
 
 itemlist_formaction::~itemlist_formaction() { }
@@ -40,10 +41,11 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 
 	switch (op) {
 		case OP_OPEN: {
-				GetLogger().log(LOG_INFO, "itemlist_formaction: opening item at pos `%s'", itemposname.c_str());
+				LOG(LOG_INFO, "itemlist_formaction: opening item at pos `%s'", itemposname.c_str());
 				if (itemposname.length() > 0) {
 					visible_items[itempos].first->set_unread(false); // set article as read
-					v->push_itemview(feed, visible_items[itempos].first->guid());
+					old_itempos = itempos;
+					v->push_itemview(feed, visible_items[itempos].first->guid(), show_searchresult ? searchphrase : "");
 					do_redraw = true;
 				} else {
 					v->show_error(_("No item selected!")); // should not happen
@@ -71,7 +73,7 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_OPENINBROWSER: {
-				GetLogger().log(LOG_INFO, "itemlist_formaction: opening item at pos `%s'", itemposname.c_str());
+				LOG(LOG_INFO, "itemlist_formaction: opening item at pos `%s'", itemposname.c_str());
 				if (itemposname.length() > 0) {
 					if (itempos < visible_items.size()) {
 						v->open_in_browser(visible_items[itempos].first->link());
@@ -100,7 +102,7 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_BOOKMARK: {
-				GetLogger().log(LOG_INFO, "itemlist_formaction: bookmarking item at pos `%s'", itemposname.c_str());
+				LOG(LOG_INFO, "itemlist_formaction: bookmarking item at pos `%s'", itemposname.c_str());
 				if (itemposname.length() > 0) {
 					if (itempos < visible_items.size()) {
 						if (automatic) {
@@ -140,7 +142,7 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 			break;
 		case OP_SAVE: 
 			{
-				GetLogger().log(LOG_INFO, "itemlist_formaction: saving item at pos `%s'", itemposname.c_str());
+				LOG(LOG_INFO, "itemlist_formaction: saving item at pos `%s'", itemposname.c_str());
 				if (itemposname.length() > 0) {
 					std::string filename ;
 					if (automatic) {
@@ -161,7 +163,7 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 			break;
 		case OP_RELOAD:
 			if (!show_searchresult) {
-				GetLogger().log(LOG_INFO, "itemlist_formaction: reloading current feed");
+				LOG(LOG_INFO, "itemlist_formaction: reloading current feed");
 				v->get_ctrl()->reload(pos);
 				update_visible_items = true;
 				do_redraw = true;
@@ -170,47 +172,57 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_QUIT:
-			GetLogger().log(LOG_INFO, "itemlist_formaction: quitting");
+			LOG(LOG_INFO, "itemlist_formaction: quitting");
 			v->feedlist_mark_pos_if_visible(pos);
 			feed->purge_deleted_items();
 			quit = true;
 			break;
 		case OP_NEXTUNREAD:
-			GetLogger().log(LOG_INFO, "itemlist_formaction: jumping to next unread item");
+			LOG(LOG_INFO, "itemlist_formaction: jumping to next unread item");
 			if (!jump_to_next_unread_item(false)) {
-				if (!v->get_next_unread()) {
+				if (!v->get_next_unread(this)) {
 					v->show_error(_("No unread items."));
 				}
 			}
 			break;
 		case OP_PREVUNREAD:
-			GetLogger().log(LOG_INFO, "itemlist_formaction: jumping to previous unread item");
+			LOG(LOG_INFO, "itemlist_formaction: jumping to previous unread item");
 			if (!jump_to_previous_unread_item(false)) {
-				if (!v->get_previous_unread()) {
+				if (!v->get_previous_unread(this)) {
+					v->show_error(_("No unread items."));
+				}
+			}
+			break;
+		case OP_RANDOMUNREAD:
+			if (!jump_to_random_unread_item()) {
+				if (!v->get_random_unread(this)) {
 					v->show_error(_("No unread items."));
 				}
 			}
 			break;
 		case OP_NEXTFEED:
-			if (!v->get_next_unread_feed()) {
+			if (!v->get_next_unread_feed(this)) {
 				v->show_error(_("No unread feeds."));
 			}
 			break;
 		case OP_PREVFEED:
-			if (!v->get_prev_unread_feed()) {
+			if (!v->get_prev_unread_feed(this)) {
 				v->show_error(_("No unread feeds."));
 			}
 			break;
 		case OP_MARKFEEDREAD:
-			GetLogger().log(LOG_INFO, "itemlist_formaction: marking feed read");
+			LOG(LOG_INFO, "itemlist_formaction: marking feed read");
 			v->set_status(_("Marking feed read..."));
 			try {
 				if (feed->rssurl() != "") {
 					v->get_ctrl()->mark_all_read(pos);
 				} else {
-					GetLogger().log(LOG_DEBUG, "itemlist_formaction: oh, it looks like I'm in a pseudo-feed (search result, query feed)");
-					for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=feed->items().begin();it!=feed->items().end();++it) {
-						(*it)->set_unread_nowrite_notify(false, true);
+					{
+						scope_mutex lock(&feed->item_mutex);
+						LOG(LOG_DEBUG, "itemlist_formaction: oh, it looks like I'm in a pseudo-feed (search result, query feed)");
+						for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=feed->items().begin();it!=feed->items().end();++it) {
+							(*it)->set_unread_nowrite_notify(false, true);
+						}
 					}
 					v->get_ctrl()->catchup_all(feed);
 				}
@@ -222,7 +234,7 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 			break;
 		case OP_TOGGLESHOWREAD:
 			m.parse(FILTER_UNREAD_ITEMS);
-			GetLogger().log(LOG_DEBUG, "itemlist_formaction: toggling show-read-articles");
+			LOG(LOG_DEBUG, "itemlist_formaction: toggling show-read-articles");
 			if (v->get_cfg()->get_configvalue_as_bool("show-read-articles")) {
 				v->get_cfg()->set_configvalue("show-read-articles", "no");
 				apply_filter = true;
@@ -234,6 +246,20 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 			update_visible_items = true;
 			do_redraw = true;
 			break;
+		case OP_PIPE_TO: {
+				std::vector<qna_pair> qna;
+				if (automatic) {
+					if (args->size() > 0) {
+						qna_responses.clear();
+						qna_responses.push_back((*args)[0]);
+						finished_qna(OP_PIPE_TO);
+					}
+				} else {
+					qna.push_back(qna_pair(_("Pipe article to command: "), ""));
+					this->start_qna(qna, OP_PIPE_TO, &cmdlinehistory);
+				}
+			}
+			break;
 		case OP_SEARCH: {
 				std::vector<qna_pair> qna;
 				if (automatic) {
@@ -249,7 +275,7 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_TOGGLEITEMREAD: {
-				GetLogger().log(LOG_INFO, "itemlist_formaction: toggling item read at pos `%s'", itemposname.c_str());
+				LOG(LOG_INFO, "itemlist_formaction: toggling item read at pos `%s'", itemposname.c_str());
 				if (itemposname.length() > 0) {
 					v->set_status(_("Toggling read flag for article..."));
 					try {
@@ -281,13 +307,14 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 						newfilter = (*args)[0];
 				} else {
 					newfilter = v->select_filter(v->get_ctrl()->get_filters().get_filters());
-					GetLogger().log(LOG_DEBUG,"itemlist_formaction::run: newfilters = %s", newfilter.c_str());
+					LOG(LOG_DEBUG,"itemlist_formaction::run: newfilters = %s", newfilter.c_str());
 				}
 				if (newfilter != "") {
 					filterhistory.add_line(newfilter);
 					if (newfilter.length() > 0) {
 						if (!m.parse(newfilter)) {
-							v->show_error(_("Error: couldn't parse filter command!"));
+							v->show_error(utils::strprintf(_("Error: couldn't parse filter command `%s': %s"), newfilter.c_str(), m.get_parse_error().c_str()));
+							m.parse(FILTER_UNREAD_ITEMS);
 						} else {
 							apply_filter = true;
 							update_visible_items = true;
@@ -320,6 +347,44 @@ void itemlist_formaction::process_operation(operation op, bool automatic, std::v
 			do_redraw = true;
 			save_filterpos();
 			break;
+		case OP_SORT: {
+				char c = v->confirm(_("Sort by (d)ate/(t)itle/(f)lags/(a)uthor/(l)ink/(g)uid?"), _("dtfalg"));
+				if (!c) break;
+				std::string result(1, c);
+				if (result == _("d")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "date-asc");
+				} else if (result == _("t")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "title-asc");
+				} else if (result == _("f")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "flags-asc");
+				} else if (result == _("a")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "author-asc");
+				} else if (result == _("l")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "link-asc");
+				} else if (result == _("g")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "guid-asc");
+				}
+			}
+			break;
+		case OP_REVSORT: {
+				char c = v->confirm(_("Reverse Sort by (d)ate/(t)itle/(f)lags/(a)uthor/(l)ink/(g)uid?"), _("dtfalg"));
+				if (!c) break;
+				std::string result(1, c);
+				if (result == _("d")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "date-desc");
+				} else if (result == _("t")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "title-desc");
+				} else if (result == _("f")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "flags-desc");
+				} else if (result == _("a")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "author-desc");
+				} else if (result == _("l")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "link-desc");
+				} else if (result == _("g")) {
+					v->get_cfg()->set_configvalue("article-sort-order", "guid-desc");
+				}
+			}
+			break;
 		case OP_INT_RESIZE:
 			do_redraw = true;
 			break;
@@ -347,6 +412,26 @@ void itemlist_formaction::finished_qna(operation op) {
 			qna_start_search();
 			break;
 
+		case OP_PIPE_TO: {
+				std::string itemposname = f->get("itempos");
+				unsigned int itempos = utils::to_u(itemposname);
+				if (itemposname.length() > 0) {
+					std::string cmd = qna_responses[0];
+					std::ostringstream ostr;
+					v->get_ctrl()->write_item(visible_items[itempos].first, ostr);
+					v->push_empty_formaction();
+					stfl::reset();
+					FILE * f = popen(cmd.c_str(), "w");
+					if (f) {
+						std::string data = ostr.str();
+						fwrite(data.c_str(), data.length(), 1, f);
+						pclose(f);
+					}
+					v->pop_current_formaction();
+				}
+			}
+			break;
+
 		default:
 			break;
 	}
@@ -384,13 +469,13 @@ void itemlist_formaction::qna_end_editflags() {
 		visible_items[itempos].first->set_flags(qna_responses[0]);
 		visible_items[itempos].first->update_flags();
 		v->set_status(_("Flags updated."));
-		GetLogger().log(LOG_DEBUG, "itemlist_formaction::finished_qna: updated flags");
+		LOG(LOG_DEBUG, "itemlist_formaction::finished_qna: updated flags");
 		do_redraw = true;
 	}
 }
 
 void itemlist_formaction::qna_start_search() {
-	std::string searchphrase = qna_responses[0];
+	searchphrase = qna_responses[0];
 	if (searchphrase.length() == 0)
 		return;
 
@@ -414,11 +499,15 @@ void itemlist_formaction::qna_start_search() {
 		return;
 	}
 
-	search_dummy_feed->items() = items;
+	{
+	    scope_mutex lock(&search_dummy_feed->item_mutex);
+	    search_dummy_feed->items() = items;
+	}
+
 	if (show_searchresult) {
 		v->pop_current_formaction();
 	}
-	v->push_searchresult(search_dummy_feed);
+	v->push_searchresult(search_dummy_feed, searchphrase);
 }
 
 void itemlist_formaction::do_update_visible_items() {
@@ -427,6 +516,7 @@ void itemlist_formaction::do_update_visible_items() {
 
 	update_visible_items = false;
 
+	scope_mutex lock(&feed->item_mutex);
 	std::vector<std::tr1::shared_ptr<rss_item> >& items = feed->items();
 
 	visible_items.clear();
@@ -439,12 +529,13 @@ void itemlist_formaction::do_update_visible_items() {
 
 	unsigned int i=0;
 	for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it = items.begin(); it != items.end(); ++it, ++i) {
+		(*it)->set_index(i+1);
 		if (!apply_filter || m.matches(it->get())) {
 			visible_items.push_back(itemptr_pos_pair(*it, i));
 		}
 	}
 
-	GetLogger().log(LOG_DEBUG, "itemlist_formaction::do_update_visible_items: size = %u", visible_items.size());
+	LOG(LOG_DEBUG, "itemlist_formaction::do_update_visible_items: size = %u", visible_items.size());
 
 	do_redraw = true;
 }
@@ -452,8 +543,27 @@ void itemlist_formaction::do_update_visible_items() {
 void itemlist_formaction::prepare() {
 	scope_mutex mtx(&redraw_mtx);
 
+	std::string sort_order = v->get_cfg()->get_configvalue("article-sort-order");
+	if (sort_order != old_sort_order) {
+		feed->sort(sort_order);
+		old_sort_order = sort_order;
+		update_visible_items = true;
+		do_redraw = true;
+	}
+
 	do_update_visible_items();
 
+	if (v->get_cfg()->get_configvalue_as_bool("mark-as-read-on-hover")) {
+		std::string itemposname = f->get("itempos");
+		if (itemposname.length() > 0) {
+			unsigned int itempos = utils::to_u(itemposname);
+			if (visible_items[itempos].first->unread()) {
+				visible_items[itempos].first->set_unread(false);
+				do_redraw = true;
+			}
+		}
+	}
+
 	unsigned int width = utils::to_u(f->get("items:w"));
 
 	if (old_width != width) {
@@ -521,7 +631,7 @@ void itemlist_formaction::set_head(const std::string& s, unsigned int unread, un
 	fmt.register_fmt('t', utils::to_s(total));
 
 	fmt.register_fmt('T', s);
-	fmt.register_fmt('U', url);
+	fmt.register_fmt('U', utils::censor_url(url));
 
 	if (!show_searchresult) {
 		title = fmt.do_format(v->get_cfg()->get_configvalue("articlelist-title-format"));
@@ -536,7 +646,7 @@ bool itemlist_formaction::jump_to_previous_unread_item(bool start_with_last) {
 	std::istringstream is(f->get("itempos"));
 	is >> itempos;
 	for (int i=(start_with_last?itempos:(itempos-1));i>=0;--i) {
-		GetLogger().log(LOG_DEBUG, "itemlist_formaction::jump_to_previous_unread_item: visible_items[%u] unread = %s", i, visible_items[i].first->unread() ? "true" : "false");
+		LOG(LOG_DEBUG, "itemlist_formaction::jump_to_previous_unread_item: visible_items[%u] unread = %s", i, visible_items[i].first->unread() ? "true" : "false");
 		if (visible_items[i].first->unread()) {
 			f->set("itempos", utils::to_s(i));
 			return true;
@@ -552,20 +662,40 @@ bool itemlist_formaction::jump_to_previous_unread_item(bool start_with_last) {
 
 }
 
+bool itemlist_formaction::jump_to_random_unread_item() {
+	bool has_unread_available = false;
+	for (unsigned int i=0;i<visible_items.size();++i) {
+		if (visible_items[i].first->unread()) {
+			has_unread_available = true;
+			break;
+		}
+	}
+	if (has_unread_available) {
+		for (;;) {
+			unsigned int pos = utils::get_random_value(visible_items.size());
+			if (visible_items[pos].first->unread()) {
+				f->set("itempos", utils::to_s(pos));
+				break;
+			}
+		}
+	}
+	return has_unread_available;
+}
+
 bool itemlist_formaction::jump_to_next_unread_item(bool start_with_first) {
 	unsigned int itempos;
 	std::istringstream is(f->get("itempos"));
 	is >> itempos;
-	GetLogger().log(LOG_DEBUG, "itemlist_formaction::jump_to_next_unread_item: itempos = %u visible_items.size = %u", itempos, visible_items.size());
+	LOG(LOG_DEBUG, "itemlist_formaction::jump_to_next_unread_item: itempos = %u visible_items.size = %u", itempos, visible_items.size());
 	for (unsigned int i=(start_with_first?itempos:(itempos+1));i<visible_items.size();++i) {
-		GetLogger().log(LOG_DEBUG, "itemlist_formaction::jump_to_next_unread_item: i = %u", i);
+		LOG(LOG_DEBUG, "itemlist_formaction::jump_to_next_unread_item: i = %u", i);
 		if (visible_items[i].first->unread()) {
 			f->set("itempos", utils::to_s(i));
 			return true;
 		}
 	}
 	for (unsigned int i=0;i<=itempos;++i) {
-		GetLogger().log(LOG_DEBUG, "itemlist_formaction::jump_to_next_unread_item: i = %u", i);
+		LOG(LOG_DEBUG, "itemlist_formaction::jump_to_next_unread_item: i = %u", i);
 		if (visible_items[i].first->unread()) {
 			f->set("itempos", utils::to_s(i));
 			return true;
@@ -620,7 +750,7 @@ void itemlist_formaction::handle_cmdline(const std::string& cmd) {
 		if (tokens[0] == "save" && tokens.size() >= 2) {
 			std::string filename = utils::resolve_tilde(tokens[1]);
 			std::string itemposname = f->get("itempos");
-			GetLogger().log(LOG_INFO, "itemlist_formaction::handle_cmdline: saving item at pos `%s' to `%s'", itemposname.c_str(), filename.c_str());
+			LOG(LOG_INFO, "itemlist_formaction::handle_cmdline: saving item at pos `%s' to `%s'", itemposname.c_str(), filename.c_str());
 			if (itemposname.length() > 0) {
 				std::istringstream posname(itemposname);
 				unsigned int itempos = 0;
@@ -647,6 +777,19 @@ int itemlist_formaction::get_pos(unsigned int realidx) {
 void itemlist_formaction::recalculate_form() {
 	formaction::recalculate_form();
 	set_update_visible_items(true);
+
+	std::string itemposname = f->get("itempos");
+	unsigned int itempos = utils::to_u(itemposname);
+	
+	// If the old position was set and it is less than the itempos, use it for the feed's itempos
+	// Correct the problem when you open itemview and jump to next then exit to itemlist and the itempos is wrong
+	// This only applies when "show-read-articles" is set to false
+	if ( (old_itempos != -1) && itempos > (unsigned int)old_itempos
+		&& ! v->get_cfg()->get_configvalue_as_bool("show-read-articles") )
+	{
+		f->set("itempos", utils::strprintf("%u", old_itempos));
+		old_itempos = -1; // Reset
+	}
 }
 
 void itemlist_formaction::save_article(const std::string& filename, std::tr1::shared_ptr<rss_item> item) {
@@ -724,11 +867,21 @@ void itemlist_formaction::prepare_set_filterpos() {
 }
 
 void itemlist_formaction::set_feed(std::tr1::shared_ptr<rss_feed> fd) {
-	GetLogger().log(LOG_DEBUG, "itemlist_formaction::set_feed: fd pointer = %p title = `%s'", fd.get(), fd->title().c_str());
+	LOG(LOG_DEBUG, "itemlist_formaction::set_feed: fd pointer = %p title = `%s'", fd.get(), fd->title().c_str());
 	feed = fd; 
 	update_visible_items = true; 
-	apply_filter = false;
 	do_update_visible_items();
 }
 
+std::string itemlist_formaction::title() {
+	if (feed->rssurl() == "") {
+		return utils::strprintf(_("Search Result - '%s'"), searchphrase.c_str());
+	} else {
+		if (feed->rssurl().substr(0,6) == "query:")
+			return utils::strprintf(_("Query Feed - %s"), feed->rssurl().substr(6,feed->rssurl().length()-6).c_str());
+		else
+			return utils::strprintf(_("Article List - %s"), feed->title().c_str());
+	}
+}
+
 }
