@@ -6,13 +6,17 @@
 #include <utils.h>
 #include <formatstring.h>
 #include <listformatter.h>
+#include <cstring>
 
 #include <sstream>
+#include <algorithm>
 
 namespace newsbeuter {
 
-itemview_formaction::itemview_formaction(view * vv, std::string formstr)
-	: formaction(vv,formstr), show_source(false), quit(false), rxman(0), num_lines(0) { 
+itemview_formaction::itemview_formaction(view * vv, std::tr1::shared_ptr<itemlist_formaction> il, std::string formstr)
+	: formaction(vv,formstr), show_source(false), quit(false), rxman(0), num_lines(0), itemlist(il), in_search(false) { 
+	valid_cmds.push_back("save");
+	std::sort(valid_cmds.begin(), valid_cmds.end());
 }
 
 itemview_formaction::~itemview_formaction() { }
@@ -50,7 +54,7 @@ void itemview_formaction::prepare() {
 		std::vector<std::string> lines;
 		std::string widthstr = f->get("article:w");
 		unsigned int render_width = 80;
-		unsigned int view_width;
+		unsigned int view_width = 0;
 		if (widthstr.length() > 0) {
 			view_width = render_width = utils::to_u(widthstr);
 			if (render_width - 5 > 0)
@@ -88,7 +92,7 @@ void itemview_formaction::prepare() {
 		}
 
 		if (item->link().length() > 0) {
-			std::string link = utils::strprintf("%s%s", _("Link: "), item->link().c_str());
+			std::string link = utils::strprintf("%s%s", _("Link: "), utils::censor_url(item->link()).c_str());
 			listfmt.add_line(link, UINT_MAX, view_width);
 		}
 
@@ -101,7 +105,10 @@ void itemview_formaction::prepare() {
 		}
 
 		if (item->enclosure_url().length() > 0) {
-			std::string enc_url = utils::strprintf("%s%s (%s%s)", _("Podcast Download URL: "), item->enclosure_url().c_str(), _("type: "), item->enclosure_type().c_str());
+			std::string enc_url = utils::strprintf("%s%s", _("Podcast Download URL: "), utils::censor_url(item->enclosure_url()).c_str());
+			if (item->enclosure_type() != "") {
+				enc_url.append(utils::strprintf(" (%s%s)",  _("type: "), item->enclosure_type().c_str()));
+			}
 			listfmt.add_line(enc_url, UINT_MAX, view_width);
 		}
 
@@ -115,7 +122,7 @@ void itemview_formaction::prepare() {
 		}
 
 		if (show_source) {
-			render_source(lines, item->description(), render_width);
+			render_source(lines, utils::quote_for_stfl(item->description()), render_width);
 		} else {
 			lines = render_html(item->description(), links, item->feedurl(), render_width);
 		}
@@ -127,6 +134,11 @@ void itemview_formaction::prepare() {
 		f->modify("article","replace_inner",listfmt.format_list(rxman, "article"));
 		f->set("articleoffset","0");
 
+		if (in_search) {
+			rxman->remove_last_regex("article");
+			in_search = false;
+		}
+
 		do_redraw = false;
 	}
 
@@ -153,7 +165,7 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 			// nothing
 			break;
 		case OP_TOGGLESOURCEVIEW:
-			GetLogger().log(LOG_INFO, "view::run_itemview: toggling source view");
+			LOG(LOG_INFO, "view::run_itemview: toggling source view");
 			show_source = !show_source;
 			do_redraw = true;
 			break;
@@ -166,7 +178,7 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 			break;
 		case OP_SAVE:
 			{
-				GetLogger().log(LOG_INFO, "view::run_itemview: saving article");
+				LOG(LOG_INFO, "view::run_itemview: saving article");
 				std::string filename;
 				if (automatic) {
 					if (args->size() > 0)
@@ -187,7 +199,7 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_OPENINBROWSER:
-			GetLogger().log(LOG_INFO, "view::run_itemview: starting browser");
+			LOG(LOG_INFO, "view::run_itemview: starting browser");
 			v->set_status(_("Starting browser..."));
 			v->open_in_browser(item->link());
 			v->set_status("");
@@ -202,6 +214,34 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 				this->start_bookmark_qna(item->title(), item->link(), "");
 			}
 			break;
+		case OP_SEARCH: {
+				std::vector<qna_pair> qna;
+				if (automatic) {
+					if (args->size() > 0) {
+						qna_responses.clear();
+						qna_responses.push_back((*args)[0]);
+						finished_qna(OP_INT_START_SEARCH);
+					}
+				} else {
+					qna.push_back(qna_pair(_("Search for: "), ""));
+					this->start_qna(qna, OP_INT_START_SEARCH, &searchhistory);
+				}
+			}
+			break;
+		case OP_PIPE_TO: {
+				std::vector<qna_pair> qna;
+				if (automatic) {
+					if (args->size() > 0) {
+						qna_responses.clear();
+						qna_responses.push_back((*args)[0]);
+						finished_qna(OP_PIPE_TO);
+					}
+				} else {
+					qna.push_back(qna_pair(_("Pipe article to command: "), ""));
+					this->start_qna(qna, OP_PIPE_TO, &cmdlinehistory);
+				}
+			}
+			break;
 		case OP_EDITFLAGS: 
 			if (automatic) {
 				qna_responses.clear();
@@ -216,7 +256,7 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_SHOWURLS:
-			GetLogger().log(LOG_DEBUG, "view::run_itemview: showing URLs");
+			LOG(LOG_DEBUG, "view::run_itemview: showing URLs");
 			if (links.size() > 0) {
 				v->push_urlview(links);
 			} else {
@@ -224,8 +264,8 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_NEXTUNREAD:
-			GetLogger().log(LOG_INFO, "view::run_itemview: jumping to next unread article");
-			if (v->get_next_unread()) {
+			LOG(LOG_INFO, "view::run_itemview: jumping to next unread article");
+			if (v->get_next_unread(itemlist.get(), this)) {
 				do_redraw = true;
 			} else {
 				v->pop_current_formaction();
@@ -233,8 +273,17 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_PREVUNREAD:
-			GetLogger().log(LOG_INFO, "view::run_itemview: jumping to previous unread article");
-			if (v->get_previous_unread()) {
+			LOG(LOG_INFO, "view::run_itemview: jumping to previous unread article");
+			if (v->get_previous_unread(itemlist.get(), this)) {
+				do_redraw = true;
+			} else {
+				v->pop_current_formaction();
+				v->show_error(_("No unread items."));
+			}
+			break;
+		case OP_RANDOMUNREAD:
+			LOG(LOG_INFO, "view::run_itemview: jumping to random unread article");
+			if (v->get_random_unread(itemlist.get(), this)) {
 				do_redraw = true;
 			} else {
 				v->pop_current_formaction();
@@ -242,7 +291,7 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 			}
 			break;
 		case OP_TOGGLEITEMREAD:
-			GetLogger().log(LOG_INFO, "view::run_itemview: setting unread and quitting");
+			LOG(LOG_INFO, "view::run_itemview: setting unread and quitting");
 			v->set_status(_("Toggling read flag for article..."));
 			try {
 				item->set_unread(true);
@@ -253,7 +302,7 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 			quit = true;
 			break;
 		case OP_QUIT:
-			GetLogger().log(LOG_INFO, "view::run_itemview: quitting");
+			LOG(LOG_INFO, "view::run_itemview: quitting");
 			quit = true;
 			break;
 		case OP_HELP:
@@ -270,7 +319,7 @@ void itemview_formaction::process_operation(operation op, bool automatic, std::v
 		case OP_9:
 		case OP_0: {
 				unsigned int idx = op - OP_1;
-				GetLogger().log(LOG_DEBUG, "itemview::run: OP_1 = %d op = %d idx = %u", OP_1, op, idx);
+				LOG(LOG_DEBUG, "itemview::run: OP_1 = %d op = %d idx = %u", OP_1, op, idx);
 				if(idx < links.size()) {
 					v->set_status(_("Starting browser..."));
 					v->open_in_browser(links[idx].first);
@@ -385,6 +434,24 @@ void itemview_formaction::finished_qna(operation op) {
 			v->set_status(_("Flags updated."));
 			do_redraw = true;
 			break;
+		case OP_INT_START_SEARCH:
+			do_search();
+			break;
+		case OP_PIPE_TO: {
+				std::string cmd = qna_responses[0];
+				std::ostringstream ostr;
+				v->get_ctrl()->write_item(feed->get_item_by_guid(guid), ostr);
+				v->push_empty_formaction();
+				stfl::reset();
+				FILE * f = popen(cmd.c_str(), "w");
+				if (f) {
+					std::string data = ostr.str();
+					fwrite(data.c_str(), data.length(), 1, f);
+					pclose(f);
+				}
+				v->pop_current_formaction();
+			}
+			break;
 		default:
 			break;
 	}
@@ -402,12 +469,15 @@ std::vector<std::string> itemview_formaction::render_html(const std::string& sou
 		argv[1] = const_cast<char *>("-c");
 		argv[2] = const_cast<char *>(renderer.c_str());
 		argv[3] = NULL;
+		LOG(LOG_DEBUG, "itemview_formaction::render_html: source = %s", source.c_str());
+		LOG(LOG_DEBUG, "itemview_formaction::render_html: html-renderer = %s", argv[2]);
+
 		std::string output = utils::run_program(argv, source);
 		std::istringstream is(output);
 		std::string line;
 		getline(is, line);
 		while (!is.eof()) {
-			lines.push_back(line);
+			lines.push_back(utils::quote_for_stfl(line));
 			getline(is, line);
 		}
 	}
@@ -422,6 +492,7 @@ void itemview_formaction::set_regexmanager(regexmanager * r) {
 	for (std::vector<std::string>::iterator it=attrs.begin();it!=attrs.end();++it,++i) {
 		attrstr.append(utils::strprintf("@style_%u_normal:%s ", i, it->c_str()));
 	}
+	attrstr.append("@style_b_normal:attr=bold @style_u_normal:attr=underline ");
 	std::string textview = utils::strprintf("{textview[article] style_normal[article]: style_end[styleend]:fg=blue,attr=bold %s .expand:vh offset[articleoffset]:0 richtext:1}", attrstr.c_str());
 	f->modify("article", "replace", textview);
 }
@@ -438,7 +509,7 @@ void itemview_formaction::update_percent() {
 		else
 			percent = 0;
 
-		GetLogger().log(LOG_DEBUG, "itemview_formaction::update_percent: offset = %u num_lines = %u percent = %u", offset, num_lines, percent);
+		LOG(LOG_DEBUG, "itemview_formaction::update_percent: offset = %u num_lines = %u percent = %u", offset, num_lines, percent);
 
 		if (offset == 0 || percent == 0) {
 			f->set("percent", _("Top"));
@@ -450,4 +521,48 @@ void itemview_formaction::update_percent() {
 	}
 }
 
+std::string itemview_formaction::title() {
+	std::tr1::shared_ptr<rss_item> item = feed->get_item_by_guid(guid);
+	return utils::strprintf(_("Article - %s"), item->title().c_str());
+}
+
+void itemview_formaction::set_highlightphrase(const std::string& text) {
+	highlight_text(text);
+}
+
+void itemview_formaction::do_search() {
+	std::string searchphrase = qna_responses[0];
+	if (searchphrase.length() == 0)
+		return;
+
+	searchhistory.add_line(searchphrase);
+
+	LOG(LOG_DEBUG, "itemview_formaction::do_search: searchphrase = %s", searchphrase.c_str());
+
+	highlight_text(searchphrase);
+}
+
+void itemview_formaction::highlight_text(const std::string& searchphrase) {
+	std::vector<std::string> params;
+	params.push_back("article");
+	params.push_back(searchphrase);
+
+	std::vector<std::string> colors = utils::tokenize(v->get_cfg()->get_configvalue("search-highlight-colors"), " ");
+	std::copy(colors.begin(), colors.end(), std::back_inserter(params));
+
+	try {
+		rxman->handle_action("highlight", params);
+
+		LOG(LOG_DEBUG, "itemview_formaction::highlight_text: configuration manipulation was successful");
+
+		set_regexmanager(rxman);
+
+		in_search = true;
+		do_redraw = true;
+	} catch (const confighandlerexception& e) {
+		LOG(LOG_ERROR, "itemview_formaction::highlight_text: handle_action failed, error = %s", e.what());
+		v->show_error(_("Error: invalid regular expression!"));
+	}
+}
+
 }
