@@ -3,6 +3,8 @@
 #include <vector>
 #include <iostream>
 #include <config.h>
+#include <exceptions.h>
+#include <utils.h>
 
 namespace newsbeuter {
 	
@@ -18,8 +20,8 @@ struct op_desc {
  * This is the list of operations, defining operation, operation name (for keybindings), default key, description, and where it's valid
  */
 static op_desc opdescs[] = {
-	{ OP_OPEN,				"open",						"ENTER",_("Open feed/article"),					KM_FEEDLIST | KM_FILEBROWSER | KM_HELP | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT | KM_URLVIEW | KM_PODBEUTER },
-	{ OP_QUIT,				"quit",						"q",	_("Return to previous dialog/Quit"),	KM_FEEDLIST | KM_FILEBROWSER | KM_HELP | KM_ARTICLELIST | KM_ARTICLE | KM_TAGSELECT | KM_FILTERSELECT | KM_URLVIEW | KM_PODBEUTER },
+	{ OP_OPEN,				"open",						"ENTER", _("Open feed/article"),				KM_FEEDLIST | KM_FILEBROWSER | KM_HELP | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT | KM_URLVIEW | KM_PODBEUTER | KM_DIALOGS },
+	{ OP_QUIT,				"quit",						"q",	_("Return to previous dialog/Quit"),	KM_FEEDLIST | KM_FILEBROWSER | KM_HELP | KM_ARTICLELIST | KM_ARTICLE | KM_TAGSELECT | KM_FILTERSELECT | KM_URLVIEW | KM_PODBEUTER | KM_DIALOGS },
 	{ OP_RELOAD,			"reload",					"r",	_("Reload currently selected feed"),	KM_FEEDLIST }, 
 	{ OP_RELOADALL,			"reload-all",				"R",	_("Reload all feeds"),					KM_FEEDLIST },
 	{ OP_MARKFEEDREAD,		"mark-feed-read",			"A",	_("Mark feed read"),					KM_FEEDLIST | KM_ARTICLELIST },
@@ -27,6 +29,7 @@ static op_desc opdescs[] = {
 	{ OP_SAVE,				"save",						"s",	_("Save article"),						KM_ARTICLELIST | KM_ARTICLE },
 	{ OP_NEXTUNREAD,		"next-unread",				"n",	_("Go to next unread article"),			KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE },
 	{ OP_PREVUNREAD,		"prev-unread",				"p",	_("Go to previous unread article"),		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE },
+	{ OP_RANDOMUNREAD,		"random-unread",			"^K",	_("Go to a random unread article"),		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE },
 	{ OP_OPENINBROWSER,		"open-in-browser",			"o",	_("Open article in browser"),			KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE },
 	{ OP_HELP,				"help",						"?",	_("Open help dialog"),					KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE | KM_PODBEUTER },
 	{ OP_TOGGLESOURCEVIEW,	"toggle-source-view",		"^U",	_("Toggle source view"),				KM_ARTICLE },
@@ -35,7 +38,7 @@ static op_desc opdescs[] = {
 	{ OP_SHOWURLS,			"show-urls",				"u",	_("Show URLs in current article"),		KM_ARTICLE | KM_ARTICLELIST },
 	{ OP_CLEARTAG,			"clear-tag",				"^T",	_("Clear current tag"),					KM_FEEDLIST },
 	{ OP_SETTAG,			"set-tag",					"t",	_("Select tag"),						KM_FEEDLIST },
-	{ OP_SEARCH,			"open-search",				"/",	_("Open search dialog"),				KM_FEEDLIST | KM_HELP | KM_ARTICLELIST },
+	{ OP_SEARCH,			"open-search",				"/",	_("Open search dialog"),				KM_FEEDLIST | KM_HELP | KM_ARTICLELIST | KM_ARTICLE },
 	{ OP_ENQUEUE,			"enqueue",					"e",	_("Add download to queue"),				KM_ARTICLE },
 	{ OP_RELOADURLS,		"reload-urls",				"^R",	_("Reload the list of URLs from the configuration"),	KM_ARTICLELIST },
 	{ OP_PB_DOWNLOAD,		"pb-download",				"d",	_("Download file"),						KM_PODBEUTER },
@@ -46,7 +49,7 @@ static op_desc opdescs[] = {
 	{ OP_PB_PLAY,			"pb-play",					"p",	_("Start player with currently selected download"),		KM_PODBEUTER },
 	{ OP_PB_MOREDL,			"pb-increase-max-dls",		"+",	_("Increase the number of concurrent downloads"),		KM_PODBEUTER },
 	{ OP_PB_LESSDL,			"pb-decreate-max-dls",		"-",	_("Decrease the number of concurrent downloads"),		KM_PODBEUTER },
-	{ OP_REDRAW,			"redraw",					"^L",	_("Redraw screen"),						KM_BOTH },
+	{ OP_REDRAW,			"redraw",					"^L",	_("Redraw screen"),						KM_SYSKEYS },
 	{ OP_CMDLINE,			"cmdline",					":",	_("Open the commandline"),				KM_NEWSBEUTER },
 	{ OP_SETFILTER,			"set-filter",				"F",	_("Set a filter"),						KM_FEEDLIST | KM_ARTICLELIST },
 	{ OP_SELECTFILTER,		"select-filter",			"f",	_("Select a predefined filter"),		KM_FEEDLIST | KM_ARTICLELIST },
@@ -59,6 +62,13 @@ static op_desc opdescs[] = {
 	{ OP_DELETE,			"delete-article",			"D",	_("Delete article"),					KM_ARTICLELIST },
 	{ OP_PURGE_DELETED,		"purge-deleted",			"$",	_("Purge deleted articles"),			KM_ARTICLELIST },
 	{ OP_EDIT_URLS,			"edit-urls",				"E",	_("Edit subscribed URLs"),				KM_FEEDLIST },
+	{ OP_CLOSEDIALOG,		"close-dialog",				"^X",	_("Close currently selected dialog"),	KM_DIALOGS },
+	{ OP_VIEWDIALOGS,		"view-dialogs",				"v",	_("View list of open dialogs"),			KM_NEWSBEUTER },
+	{ OP_NEXTDIALOG,		"next-dialog",				"^V",	_("Go to next dialog"),					KM_NEWSBEUTER },
+	{ OP_PREVDIALOG,		"prev-dialog",				"^G",	_("Go to previous dialog"),				KM_NEWSBEUTER },
+	{ OP_PIPE_TO,			"pipe-to",					"|",	_("Pipe article to command"),			KM_ARTICLE | KM_ARTICLELIST },
+	{ OP_SORT,				"sort",						"g",	_("Sort current list"),					KM_FEEDLIST | KM_ARTICLELIST },
+	{ OP_REVSORT,			"rev-sort",					"G",	_("Sort current list (reverse)"),		KM_FEEDLIST | KM_ARTICLELIST },
 
 	{ OP_0,		"zero",	"0",	_("Open URL 10"), KM_URLVIEW | KM_ARTICLE },
 	{ OP_1,		"one",	"1",	_("Open URL 1"), KM_URLVIEW | KM_ARTICLE},
@@ -71,10 +81,10 @@ static op_desc opdescs[] = {
 	{ OP_8,		"eight","8",	_("Open URL 8"), KM_URLVIEW | KM_ARTICLE},
 	{ OP_9,		"nine",	"9",	_("Open URL 9"), KM_URLVIEW | KM_ARTICLE},
 
-	{ OP_SK_UP, "up", "UP", NULL, KM_SYSKEYS },
-	{ OP_SK_DOWN, "down", "DOWN", NULL, KM_SYSKEYS },
-	{ OP_SK_PGUP, "pageup", "PAGEUP", NULL, KM_SYSKEYS },
-	{ OP_SK_PGDOWN, "pagedown", "PAGEDOWN", NULL, KM_SYSKEYS },
+	{ OP_SK_UP, "up", "UP", _("Move to the previous entry"), KM_SYSKEYS },
+	{ OP_SK_DOWN, "down", "DOWN", _("Move to the next entry"), KM_SYSKEYS },
+	{ OP_SK_PGUP, "pageup", "PAGEUP", _("Move to the previous page"), KM_SYSKEYS },
+	{ OP_SK_PGDOWN, "pagedown", "PAGEDOWN", _("Move to the next page"), KM_SYSKEYS },
 
 	{ OP_INT_END_QUESTION, "XXXNOKEY-end-question", "end-question", NULL, KM_INTERNAL },
 	{ OP_INT_CANCEL_QNA, "XXXNOKEY-cancel-qna", "cancel-qna", NULL, KM_INTERNAL },
@@ -88,19 +98,18 @@ static op_desc opdescs[] = {
 };
 
 // "all" must be first, the following positions must be the same as the KM_* flag definitions (get_flag_from_context() relies on this).
-static const char * contexts[] = { "all", "feedlist", "filebrowser", "help", "articlelist", "article", "tagselection", "filterselection", "urlview", "podbeuter", NULL };
+static const char * contexts[] = { "all", "feedlist", "filebrowser", "help", "articlelist", "article", "tagselection", "filterselection", "urlview", "podbeuter", "dialogs", NULL };
 
 keymap::keymap(unsigned flags) { 
 	/*
 	 * At startup, initialize the keymap with the default settings from the list above.
 	 */
-	GetLogger().log(LOG_DEBUG, "keymap::keymap: flags = %x", flags);
+	LOG(LOG_DEBUG, "keymap::keymap: flags = %x", flags);
 	for (unsigned int j=1;contexts[j]!=NULL;j++) {
 		std::string ctx(contexts[j]);
 		for (int i=0;opdescs[i].op != OP_NIL;++i) {
 			if (opdescs[i].flags & (flags | KM_INTERNAL | KM_SYSKEYS)) {
 				keymap_[ctx][opdescs[i].default_key] = opdescs[i].op;
-				GetLogger().log(LOG_DEBUG, "keymap::keymap: keymap_[%s][%s] = %u (%s)", ctx.c_str(), opdescs[i].default_key, opdescs[i].op, opdescs[i].opstr);
 			}
 		}
 	}
@@ -114,30 +123,44 @@ void keymap::get_keymap_descriptions(std::vector<keymap_desc>& descs, unsigned s
 	for (unsigned int i=1;contexts[i]!=NULL;i++) {
 		std::string ctx(contexts[i]);
 
-		if (flags & KM_PODBEUTER && ctx != "podbeuter" && ctx != "help") {
+		if (flags & KM_PODBEUTER && ctx != "podbeuter") {
 			continue;
 		} else if (flags & KM_NEWSBEUTER && ctx == "podbeuter") {
 			continue;
 		}
 
-		for (int i=0;opdescs[i].help_text;++i) {
+		for (unsigned int j=0;opdescs[j].op != OP_NIL;++j) {
 			bool already_added = false;
 			for (std::map<std::string,operation>::iterator it=keymap_[ctx].begin();it!=keymap_[ctx].end();++it) {
 				operation op = it->second;
 				if (op != OP_NIL) {
-					if (opdescs[i].op == op && opdescs[i].flags & flags) {
+					if (opdescs[j].op == op && opdescs[j].flags & flags) {
 						keymap_desc desc;
 						desc.key = it->first;
 						desc.ctx = ctx;
 						if (!already_added) {
-							desc.cmd = opdescs[i].opstr;
-							desc.desc = gettext(opdescs[i].help_text);
+							desc.cmd = opdescs[j].opstr;
+							if (opdescs[j].help_text)
+								desc.desc = gettext(opdescs[j].help_text);
 							already_added = true;
 						}
+						desc.flags = opdescs[j].flags;
 						descs.push_back(desc);
 					}
 				}
 			}
+			if (!already_added) {
+				if (opdescs[j].flags & flags) {
+					LOG(LOG_DEBUG, "keymap::get_keymap_descriptions: found unbound function: %s ctx = %s", opdescs[j].opstr, ctx.c_str());
+					keymap_desc desc;
+					desc.ctx = ctx;
+					desc.cmd = opdescs[j].opstr;
+					if (opdescs[j].help_text)
+						desc.desc = gettext(opdescs[j].help_text);
+					desc.flags = opdescs[j].flags;
+					descs.push_back(desc);
+				}
+			}
 		}
 	}
 }
@@ -146,7 +169,7 @@ keymap::~keymap() { }
 
 
 void keymap::set_key(operation op, const std::string& key, const std::string& context) {
-	GetLogger().log(LOG_DEBUG,"keymap::set_key(%d,%s) called", op, key.c_str());
+	LOG(LOG_DEBUG,"keymap::set_key(%d,%s) called", op, key.c_str());
 	if (context == "all") {
 		for (unsigned int i=0;contexts[i]!=NULL;i++) {
 			keymap_[contexts[i]][key] = op;
@@ -157,7 +180,7 @@ void keymap::set_key(operation op, const std::string& key, const std::string& co
 }
 
 void keymap::unset_key(const std::string& key, const std::string& context) {
-	GetLogger().log(LOG_DEBUG,"keymap::unset_key(%s) called", key.c_str());
+	LOG(LOG_DEBUG,"keymap::unset_key(%s) called", key.c_str());
 	if (context == "all") {
 		for (unsigned int i=0;contexts[i]!=NULL;i++) {
 			keymap_[contexts[i]][key] = OP_NIL;
@@ -192,7 +215,7 @@ char keymap::get_key(const std::string& keycode) {
 
 operation keymap::get_operation(const std::string& keycode, const std::string& context) {
 	std::string key;
-	GetLogger().log(LOG_DEBUG, "keymap::get_operation: keycode = %s context = %s", keycode.c_str(), context.c_str());
+	LOG(LOG_DEBUG, "keymap::get_operation: keycode = %s context = %s", keycode.c_str(), context.c_str());
 	if (keycode.length() > 0) {
 		key = keycode;
 	} else {
@@ -201,41 +224,75 @@ operation keymap::get_operation(const std::string& keycode, const std::string& c
 	return keymap_[context][key];
 }
 
-action_handler_status keymap::handle_action(const std::string& action, const std::vector<std::string>& params) {
+void keymap::dump_config(std::vector<std::string>& config_output) {
+	for (unsigned int i=1;contexts[i]!=NULL;i++) { // TODO: optimize
+		std::map<std::string,operation>& x = keymap_[contexts[i]];
+		for (std::map<std::string,operation>::iterator it = x.begin();it!=x.end();it++) {
+			if (it->second < OP_INT_MIN) {
+				std::string configline = "bind-key ";
+				configline.append(it->first);
+				configline.append(" ");
+				configline.append(getopname(it->second));
+				configline.append(" ");
+				configline.append(contexts[i]);
+				config_output.push_back(configline);
+			}
+		}
+	}
+	for (std::map<std::string,std::vector<macrocmd> >::iterator it=macros_.begin();it!=macros_.end();it++) {
+		std::string configline = "macro ";
+		configline.append(it->first);
+		configline.append(" ");
+		unsigned int i=0;
+		for (std::vector<macrocmd>::iterator jt=it->second.begin();jt!=it->second.end();jt++,i++) {
+			configline.append(getopname(jt->op));
+			for (std::vector<std::string>::iterator kt=jt->args.begin();kt!=jt->args.end();kt++) {
+				configline.append(" ");
+				configline.append(utils::quote(*kt));
+			}
+			if (i < (it->second.size()-1))
+				configline.append(" ; ");
+		}
+		config_output.push_back(configline);
+	}
+}
+
+std::string keymap::getopname(operation op) {
+	for (unsigned int i=0;opdescs[i].op != OP_NIL;i++) {
+		if (opdescs[i].op == op)
+			return opdescs[i].opstr;
+	}
+	return "<none>";
+}
+
+void keymap::handle_action(const std::string& action, const std::vector<std::string>& params) {
 	/*
 	 * The keymap acts as config_action_handler so that all the key-related configuration is immediately
 	 * handed to it.
 	 */
-	GetLogger().log(LOG_DEBUG,"keymap::handle_action(%s, ...) called",action.c_str());
+	LOG(LOG_DEBUG,"keymap::handle_action(%s, ...) called",action.c_str());
 	if (action == "bind-key") {
-		if (params.size() < 2) {
-			return AHS_TOO_FEW_PARAMS;
-		} else {
-			std::string context = "all";
-			if (params.size() >= 3)
-				context = params[2];
-			if (!is_valid_context(context))
-				return AHS_INVALID_PARAMS;
-			operation op = get_opcode(params[1]);
-			if (op > OP_SK_MIN && op < OP_SK_MAX)
-				unset_key(getkey(op, context), context);
-			set_key(op, params[0], context);
-			return AHS_OK;
-		}
+		if (params.size() < 2)
+			throw confighandlerexception(AHS_TOO_FEW_PARAMS);
+		std::string context = "all";
+		if (params.size() >= 3)
+			context = params[2];
+		if (!is_valid_context(context))
+			throw confighandlerexception(utils::strprintf(_("`%s' is not a valid context"), context.c_str()));
+		operation op = get_opcode(params[1]);
+		if (op > OP_SK_MIN && op < OP_SK_MAX)
+			unset_key(getkey(op, context), context);
+		set_key(op, params[0], context);
 	} else if (action == "unbind-key") {
-		if (params.size() < 1) {
-			return AHS_TOO_FEW_PARAMS;
-		} else {
-			std::string context = "all";
-			if (params.size() >= 2)
-				context = params[1];
-			unset_key(params[0], context);
-			return AHS_OK;	
-		}
+		if (params.size() < 1)
+			throw confighandlerexception(AHS_TOO_FEW_PARAMS);
+		std::string context = "all";
+		if (params.size() >= 2)
+			context = params[1];
+		unset_key(params[0], context);
 	} else if (action == "macro") {
-		if (params.size() < 1) {
-			return AHS_TOO_FEW_PARAMS;
-		}
+		if (params.size() < 1)
+			throw confighandlerexception(AHS_TOO_FEW_PARAMS);
 		std::vector<std::string>::const_iterator it = params.begin();
 		std::string macrokey = *it;
 		std::vector<macrocmd> cmds;
@@ -247,9 +304,9 @@ action_handler_status keymap::handle_action(const std::string& action, const std
 		while (it != params.end()) {
 			if (first && *it != ";") {
 				tmpcmd.op = get_opcode(*it);
-				GetLogger().log(LOG_DEBUG, "keymap::handle_action: new operation `%s' (op = %u)", it->c_str(), tmpcmd.op);
+				LOG(LOG_DEBUG, "keymap::handle_action: new operation `%s' (op = %u)", it->c_str(), tmpcmd.op);
 				if (tmpcmd.op == OP_NIL)
-					return AHS_INVALID_PARAMS;
+					throw confighandlerexception(utils::strprintf(_("`%s' is not a valid key command"), it->c_str()));
 				first = false;
 			} else {
 				if (*it == ";") {
@@ -259,7 +316,7 @@ action_handler_status keymap::handle_action(const std::string& action, const std
 					tmpcmd.args.clear();
 					first = true;
 				} else {
-					GetLogger().log(LOG_DEBUG, "keymap::handle_action: new parameter `%s' (op = %u)", it->c_str());
+					LOG(LOG_DEBUG, "keymap::handle_action: new parameter `%s' (op = %u)", it->c_str());
 					tmpcmd.args.push_back(*it);
 				}
 			}
@@ -269,9 +326,8 @@ action_handler_status keymap::handle_action(const std::string& action, const std
 			cmds.push_back(tmpcmd);
 
 		macros_[macrokey] = cmds;
-		return AHS_OK;
 	} else
-		return AHS_INVALID_PARAMS;
+		throw confighandlerexception(AHS_INVALID_PARAMS);
 }
 
 std::string keymap::getkey(operation op, const std::string& context) {
@@ -313,7 +369,7 @@ bool keymap::is_valid_context(const std::string& context) {
 unsigned short keymap::get_flag_from_context(const std::string& context) {
 	for (unsigned int i=1;contexts[i]!=NULL;i++) {
 		if (context == contexts[i])
-			return 1<<(i-1);
+			return (1<<(i-1)) | KM_SYSKEYS;
 	}
 	return 0; // shouldn't happen
 }
