@@ -1,9 +1,10 @@
 #include <mutex.h>
 #include <exception.h>
+#include <logger.h>
 
 #include <cerrno>
 
-using namespace newsbeuter;
+namespace newsbeuter {
 
 mutex::mutex() {
 	pthread_mutexattr_init(&attr);
@@ -26,7 +27,7 @@ void mutex::lock() {
 void mutex::unlock() {
 	int rc = pthread_mutex_unlock(&mtx);
 	if (rc != 0) {
-		throw exception(rc);
+		LOG(LOG_INFO, "mutex::unlock: unlock returned %d", rc);
 	}
 }
 
@@ -54,3 +55,5 @@ scope_mutex::~scope_mutex() {
 		mtx->unlock();
 	}
 }
+
+}
