@@ -29,31 +29,28 @@ void poddlthread::run() {
 	++bytecount;
 
 	CURL * easyhandle = curl_easy_init();
-
-	std::string user_agent = utils::get_useragent(cfg);
-
-	curl_easy_setopt(easyhandle, CURLOPT_USERAGENT, user_agent.c_str());
+	utils::set_common_curl_options(easyhandle, cfg);
 
 	curl_easy_setopt(easyhandle, CURLOPT_URL, dl->url());
+	curl_easy_setopt(easyhandle, CURLOPT_TIMEOUT, 0);
 	// set up write functions:
 	curl_easy_setopt(easyhandle, CURLOPT_WRITEFUNCTION, my_write_data);
 	curl_easy_setopt(easyhandle, CURLOPT_WRITEDATA, this);
 
 	// set up progress notification:
 	curl_easy_setopt(easyhandle, CURLOPT_NOPROGRESS, 0);
-	curl_easy_setopt(easyhandle, CURLOPT_FOLLOWLOCATION, 1);
 	curl_easy_setopt(easyhandle, CURLOPT_PROGRESSFUNCTION, progress_callback);
 	curl_easy_setopt(easyhandle, CURLOPT_PROGRESSDATA, this);
 
 	struct stat sb;
 
 	if (stat(dl->filename(), &sb) == -1) {
-		GetLogger().log(LOG_INFO, "poddlthread::run: stat failed: starting normal download");
+		LOG(LOG_INFO, "poddlthread::run: stat failed: starting normal download");
 		mkdir_p(dl->filename());
 		f.open(dl->filename(), std::fstream::out);
 		dl->set_offset(0);
 	} else {
-		GetLogger().log(LOG_INFO, "poddlthread::run: stat ok: starting download from %u", sb.st_size);
+		LOG(LOG_INFO, "poddlthread::run: stat ok: starting download from %u", sb.st_size);
 		curl_easy_setopt(easyhandle, CURLOPT_RESUME_FROM, sb.st_size);
 		dl->set_offset(sb.st_size);
 		f.open(dl->filename(), std::fstream::out | std::fstream::app);
@@ -67,7 +64,7 @@ void poddlthread::run() {
 
 		f.close();
 
-		GetLogger().log(LOG_INFO,"poddlthread::run: curl_easy_perform rc = %u", success);
+		LOG(LOG_INFO,"poddlthread::run: curl_easy_perform rc = %u (%s)", success, curl_easy_strerror(success));
 
 		if (0 == success)
 			dl->set_status(DL_FINISHED);
@@ -97,6 +94,7 @@ size_t poddlthread::write_data(void * buffer, size_t size, size_t nmemb) {
 		return 0;
 	f.write(static_cast<char *>(buffer), size * nmemb);
 	bytecount += (size * nmemb);
+	LOG(LOG_DEBUG, "poddlthread::write_data: bad = %u size = %u", f.bad(), size * nmemb);
 	return f.bad() ? 0 : size * nmemb;
 }
 
