@@ -20,26 +20,26 @@
 
 #include <functional>
 
-using namespace newsbeuter;
+namespace newsbeuter {
 
-rss_item::rss_item(cache * c) : unread_(true), ch(c), enqueued_(false), deleted_(0) {
-	// GetLogger().log(LOG_CRITICAL, "new rss_item");
+rss_item::rss_item(cache * c) : unread_(true), ch(c), enqueued_(false), deleted_(0), idx(0) {
+	// LOG(LOG_CRITICAL, "new rss_item");
 }
 
 rss_item::~rss_item() {
-	// GetLogger().log(LOG_CRITICAL, "delete rss_item");
+	// LOG(LOG_CRITICAL, "delete rss_item");
 }
 
-rss_feed::rss_feed(cache * c) : ch(c), empty(true), is_rtl_(false) {
-	// GetLogger().log(LOG_CRITICAL, "new rss_feed");
+rss_feed::rss_feed(cache * c) : ch(c), empty(true), is_rtl_(false), idx(0) {
+	// LOG(LOG_CRITICAL, "new rss_feed");
 }
 
 rss_feed::rss_feed() : ch(NULL), empty(true), is_rtl_(false) { 
-	// GetLogger().log(LOG_CRITICAL, "new rss_feed");
+	// LOG(LOG_CRITICAL, "new rss_feed");
 }
 
 rss_feed::~rss_feed() {
-	// GetLogger().log(LOG_CRITICAL, "delete rss_feed");
+	// LOG(LOG_CRITICAL, "delete rss_feed");
 }
 
 // rss_item setters
@@ -102,7 +102,8 @@ std::string rss_item::pubDate() const {
 	return std::string(text);
 }
 
-unsigned int rss_feed::unread_item_count() const {
+unsigned int rss_feed::unread_item_count() {
+	// scope_mutex lock(&item_mutex);
 	unsigned int count = 0;
 	for (std::vector<std::tr1::shared_ptr<rss_item> >::const_iterator it=items_.begin();it!=items_.end();++it) {
 		if ((*it)->unread())
@@ -153,11 +154,9 @@ void rss_item::set_enclosure_type(const std::string& type) {
 }
 
 std::string rss_item::title() const {
-	GetLogger().log(LOG_DEBUG,"rss_item::title: title before conversion: %s", title_.c_str());
 	std::string retval;
 	if (title_.length()>0)
 		retval = utils::convert_text(title_, nl_langinfo(CODESET), "utf-8");
-	GetLogger().log(LOG_DEBUG,"rss_item::title: title after conversion: %s", retval.c_str());
 	return retval;
 }
 
@@ -187,18 +186,19 @@ std::string rss_feed::description() const {
 }
 
 std::tr1::shared_ptr<rss_item> rss_feed::get_item_by_guid(const std::string& guid) {
+	scope_mutex lock(&item_mutex);
 	for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=items_.begin();it!=items_.end();++it) {
 		if ((*it)->guid() == guid) {
 			return *it;
 		}
 	}
-	GetLogger().log(LOG_DEBUG, "rss_feed::get_item_by_guid: hit dummy item!");
+	LOG(LOG_DEBUG, "rss_feed::get_item_by_guid: hit dummy item!");
 	// abort();
 	return std::tr1::shared_ptr<rss_item>(new rss_item(ch)); // should never happen!
 }
 
 bool rss_item::has_attribute(const std::string& attribname) {
-	// GetLogger().log(LOG_DEBUG, "rss_item::has_attribute(%s) called", attribname.c_str());
+	// LOG(LOG_DEBUG, "rss_item::has_attribute(%s) called", attribname.c_str());
 	if (attribname == "title" || 
 		attribname == "link" || 
 		attribname == "author" || 
@@ -208,7 +208,9 @@ bool rss_item::has_attribute(const std::string& attribname) {
 		attribname == "unread" ||
 		attribname == "enclosure_url" ||
 		attribname == "enclosure_type" ||
-		attribname == "flags")
+		attribname == "flags" ||
+		attribname == "age" ||
+		attribname == "articleindex")
 			return true;
 
 	// if we have a feed, then forward the request
@@ -239,6 +241,10 @@ std::string rss_item::get_attribute(const std::string& attribname) {
 		return enclosure_type();
 	else if (attribname == "flags")
 		return flags();
+	else if (attribname == "age")
+		return utils::to_s((time(NULL) - pubDate_timestamp()) / 86400);
+	else if (attribname == "articleindex")
+		return utils::to_s(idx);
 
 	// if we have a feed, then forward the request
 	if (feedptr)
@@ -281,7 +287,8 @@ bool rss_feed::has_attribute(const std::string& attribname) {
 		attribname == "rssurl" ||
 		attribname == "unread_count" ||
 		attribname == "total_count" ||
-		attribname == "tags")
+		attribname == "tags" ||
+		attribname == "feedindex")
 			return true;
 	return false;
 }
@@ -303,37 +310,51 @@ std::string rss_feed::get_attribute(const std::string& attribname) {
 		return utils::to_s(items_.size());
 	} else if (attribname == "tags") {
 		return get_tags();
+	} else if (attribname == "feedindex") {
+		return utils::to_s(idx);
 	}
 	return "";
 }
 
-action_handler_status rss_ignores::handle_action(const std::string& action, const std::vector<std::string>& params) {
+void rss_ignores::handle_action(const std::string& action, const std::vector<std::string>& params) {
 	if (action == "ignore-article") {
-		if (params.size() >= 2) {
-			std::string ignore_rssurl = params[0];
-			std::string ignore_expr = params[1];
-			matcher m;
-			if (m.parse(ignore_expr)) {
-				ignores.push_back(feedurl_expr_pair(ignore_rssurl, new matcher(ignore_expr)));
-				return AHS_OK;
-			} else {
-				return AHS_INVALID_PARAMS;
-			}
-		} else {
-			return AHS_TOO_FEW_PARAMS;
-		}
+		if (params.size() < 2)
+			throw confighandlerexception(AHS_TOO_FEW_PARAMS);
+		std::string ignore_rssurl = params[0];
+		std::string ignore_expr = params[1];
+		matcher m;
+		if (!m.parse(ignore_expr))
+			throw confighandlerexception(utils::strprintf(_("couldn't parse filter expression `%s': %s"), ignore_expr.c_str(), m.get_parse_error().c_str()));
+		ignores.push_back(feedurl_expr_pair(ignore_rssurl, new matcher(ignore_expr)));
 	} else if (action == "always-download") {
 		for (std::vector<std::string>::const_iterator it=params.begin();it!=params.end();++it) {
 			ignores_lastmodified.push_back(*it);
 		}
-		return AHS_OK;
 	} else if (action == "reset-unread-on-update") {
 		for (std::vector<std::string>::const_iterator it=params.begin();it!=params.end();++it) {
 			resetflag.push_back(*it);
 		}
-		return AHS_OK;
+	} else
+		throw confighandlerexception(AHS_INVALID_COMMAND);
+}
+
+void rss_ignores::dump_config(std::vector<std::string>& config_output) {
+	for (std::vector<feedurl_expr_pair>::iterator it = ignores.begin();it!=ignores.end();it++) {
+		std::string configline = "ignore-article ";
+		if (it->first == "*")
+			configline.append("*");
+		else
+			configline.append(utils::quote(it->first));
+		configline.append(" ");
+		configline.append(utils::quote(it->second->get_expression()));
+		config_output.push_back(configline);
+	}
+	for (std::vector<std::string>::iterator it=ignores_lastmodified.begin();it!=ignores_lastmodified.end();it++) {
+		config_output.push_back(utils::strprintf("always-download %s", utils::quote(*it).c_str()));
+	}
+	for (std::vector<std::string>::iterator it=resetflag.begin();it!=resetflag.end();it++) {
+		config_output.push_back(utils::strprintf("reset-unread-on-update %s", utils::quote(*it).c_str()));
 	}
-	return AHS_INVALID_COMMAND;
 }
 
 rss_ignores::~rss_ignores() {
@@ -344,10 +365,10 @@ rss_ignores::~rss_ignores() {
 
 bool rss_ignores::matches(rss_item* item) {
 	for (std::vector<feedurl_expr_pair>::iterator it=ignores.begin();it!=ignores.end();++it) {
-		GetLogger().log(LOG_DEBUG, "rss_ignores::matches: it->first = `%s' item->feedurl = `%s'", it->first.c_str(), item->feedurl().c_str());
+		LOG(LOG_DEBUG, "rss_ignores::matches: it->first = `%s' item->feedurl = `%s'", it->first.c_str(), item->feedurl().c_str());
 		if (it->first == "*" || item->feedurl() == it->first) {
 			if (it->second->matches(item)) {
-				GetLogger().log(LOG_DEBUG, "rss_ignores::matches: found match");
+				LOG(LOG_DEBUG, "rss_ignores::matches: found match");
 				return true;
 			}
 		}
@@ -372,10 +393,11 @@ bool rss_ignores::matches_resetunread(const std::string& url) {
 }
 
 void rss_feed::update_items(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds) {
+	scope_mutex lock(&item_mutex);
 	if (query.length() == 0)
 		return;
 
-	GetLogger().log(LOG_DEBUG, "rss_feed::update_items: query = `%s'", query.c_str());
+	LOG(LOG_DEBUG, "rss_feed::update_items: query = `%s'", query.c_str());
 
 
 	struct timeval tv1, tv2, tvx;
@@ -389,7 +411,7 @@ void rss_feed::update_items(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds)
 		if ((*it)->rssurl().substr(0,6) != "query:") { // don't fetch items from other query feeds!
 			for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator jt=(*it)->items().begin();jt!=(*it)->items().end();++jt) {
 				if (m.matches(jt->get())) {
-					GetLogger().log(LOG_DEBUG, "rss_feed::update_items: matcher matches!");
+					LOG(LOG_DEBUG, "rss_feed::update_items: matcher matches!");
 					(*jt)->set_feedptr(*it);
 					items_.push_back(*jt);
 				}
@@ -404,56 +426,68 @@ void rss_feed::update_items(std::vector<std::tr1::shared_ptr<rss_feed> >& feeds)
 	gettimeofday(&tv2, NULL);
 	unsigned long diff = (((tv2.tv_sec - tv1.tv_sec) * 1000000) + tv2.tv_usec) - tv1.tv_usec;
 	unsigned long diffx = (((tv2.tv_sec - tvx.tv_sec) * 1000000) + tv2.tv_usec) - tvx.tv_usec;
-	GetLogger().log(LOG_DEBUG, "rss_feed::update_items matching took %lu.%06lu s", diff / 1000000, diff % 1000000);
-	GetLogger().log(LOG_DEBUG, "rss_feed::update_items sorting took %lu.%06lu s", diffx / 1000000, diffx % 1000000);
+	LOG(LOG_DEBUG, "rss_feed::update_items matching took %lu.%06lu s", diff / 1000000, diff % 1000000);
+	LOG(LOG_DEBUG, "rss_feed::update_items sorting took %lu.%06lu s", diffx / 1000000, diffx % 1000000);
 }
 
 void rss_feed::set_rssurl(const std::string& u) {
 	rssurl_ = u;
 	if (rssurl_.substr(0,6) == "query:") {
 		std::vector<std::string> tokens = utils::tokenize_quoted(u, ":");
-		GetLogger().log(LOG_DEBUG, "rss_feed::set_rssurl: query name = `%s' expr = `%s'", tokens[1].c_str(), tokens[2].c_str());
+		LOG(LOG_DEBUG, "rss_feed::set_rssurl: query name = `%s' expr = `%s'", tokens[1].c_str(), tokens[2].c_str());
 		set_title(tokens[1]);
 		set_query(tokens[2]);
 	}
 }
 
-struct sort_item_by_title : public std::binary_function<const std::tr1::shared_ptr<rss_item>&, const std::tr1::shared_ptr<rss_item>&, bool> {
+struct sort_item_by_title : public std::binary_function<std::tr1::shared_ptr<rss_item>, std::tr1::shared_ptr<rss_item>, bool> {
 	sort_item_by_title() { }
-	bool operator()(const std::tr1::shared_ptr<rss_item>& a, const std::tr1::shared_ptr<rss_item>& b) {
+	bool operator()(std::tr1::shared_ptr<rss_item> a, std::tr1::shared_ptr<rss_item> b) {
 		return strcasecmp(a->title().c_str(), b->title().c_str()) < 0;
 	}
 };
 
-struct sort_item_by_flags : public std::binary_function<const std::tr1::shared_ptr<rss_item>&, const std::tr1::shared_ptr<rss_item>&, bool> {
+struct sort_item_by_flags : public std::binary_function<std::tr1::shared_ptr<rss_item>, std::tr1::shared_ptr<rss_item>, bool> {
 	sort_item_by_flags() { }
-	bool operator()(const std::tr1::shared_ptr<rss_item>& a, const std::tr1::shared_ptr<rss_item>& b) {
+	bool operator()(std::tr1::shared_ptr<rss_item> a, std::tr1::shared_ptr<rss_item> b) {
 		return strcmp(a->flags().c_str(), b->flags().c_str()) < 0;
 	}
 };
 
-struct sort_item_by_author : public std::binary_function<const std::tr1::shared_ptr<rss_item>&, const std::tr1::shared_ptr<rss_item>&, bool> {
+struct sort_item_by_author : public std::binary_function<std::tr1::shared_ptr<rss_item>, std::tr1::shared_ptr<rss_item>, bool> {
 	sort_item_by_author() { }
-	bool operator()(const std::tr1::shared_ptr<rss_item>& a, const std::tr1::shared_ptr<rss_item>& b) {
+	bool operator()(std::tr1::shared_ptr<rss_item> a, std::tr1::shared_ptr<rss_item> b) {
 		return strcmp(a->author().c_str(), b->author().c_str()) < 0;
 	}
 };
 
-struct sort_item_by_link : public std::binary_function<const std::tr1::shared_ptr<rss_item>&, const std::tr1::shared_ptr<rss_item>&, bool> {
+struct sort_item_by_link : public std::binary_function<std::tr1::shared_ptr<rss_item>, std::tr1::shared_ptr<rss_item>, bool> {
 	sort_item_by_link() { }
-	bool operator()(const std::tr1::shared_ptr<rss_item>& a, const std::tr1::shared_ptr<rss_item>& b) {
+	bool operator()(std::tr1::shared_ptr<rss_item> a, std::tr1::shared_ptr<rss_item> b) {
 		return strcmp(a->link().c_str(), b->link().c_str()) < 0;
 	}
 };
 
-struct sort_item_by_guid : public std::binary_function<const std::tr1::shared_ptr<rss_item>&, const std::tr1::shared_ptr<rss_item>&, bool> {
+struct sort_item_by_guid : public std::binary_function<std::tr1::shared_ptr<rss_item>, std::tr1::shared_ptr<rss_item>, bool> {
 	sort_item_by_guid() { }
-	bool operator()(const std::tr1::shared_ptr<rss_item>& a, const std::tr1::shared_ptr<rss_item>& b) {
+	bool operator()(std::tr1::shared_ptr<rss_item> a, std::tr1::shared_ptr<rss_item> b) {
 		return strcmp(a->guid().c_str(), b->guid().c_str()) < 0;
 	}
 };
 
+struct sort_item_by_date : public std::binary_function<std::tr1::shared_ptr<rss_item>, std::tr1::shared_ptr<rss_item>, bool> {
+	sort_item_by_date() { }
+	bool operator()(std::tr1::shared_ptr<rss_item> a, std::tr1::shared_ptr<rss_item> b) {
+		return a->pubDate_timestamp()  < b->pubDate_timestamp();
+	}
+};
+
 void rss_feed::sort(const std::string& method) {
+	scope_mutex lock(&item_mutex);
+	sort_unlocked(method);
+}
+
+void rss_feed::sort_unlocked(const std::string& method) {
 	std::vector<std::string> methods = utils::tokenize(method,"-");
 	bool reverse = false;
 
@@ -467,7 +501,7 @@ void rss_feed::sort(const std::string& method) {
 		}
 	}
 
-	if (methods.size() > 0 && methods[0] != "date") {
+	if (methods.size() > 0) {
 		if (methods[0] == "title") {
 			std::stable_sort(items_.begin(), items_.end(), sort_item_by_title());
 		} else if (methods[0] == "flags") {
@@ -478,7 +512,9 @@ void rss_feed::sort(const std::string& method) {
 			std::stable_sort(items_.begin(), items_.end(), sort_item_by_link());
 		} else if (methods[0] == "guid") {
 			std::stable_sort(items_.begin(), items_.end(), sort_item_by_guid());
-		} // add new sorting methods here
+		} else if (methods[0] == "date") {
+			std::stable_sort(items_.begin(), items_.end(), sort_item_by_date());
+		}
 	}
 
 	if (reverse) {
@@ -487,6 +523,7 @@ void rss_feed::sort(const std::string& method) {
 }
 
 void rss_feed::remove_old_deleted_items() {
+	scope_mutex lock(&item_mutex);
 	std::vector<std::string> guids;
 	for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=items_.begin();it!=items_.end();++it) {
 		guids.push_back((*it)->guid());
@@ -495,6 +532,7 @@ void rss_feed::remove_old_deleted_items() {
 }
 
 void rss_feed::purge_deleted_items() {
+	scope_mutex lock(&item_mutex);
 	scope_measure m1("rss_feed::purge_deleted_items");
 	std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=items_.begin();
 	while (it!=items_.end()) {
@@ -507,7 +545,16 @@ void rss_feed::purge_deleted_items() {
 	}
 }
 
+void rss_feed::set_feedptrs(std::tr1::shared_ptr<rss_feed> self) {
+	scope_mutex lock(&item_mutex);
+	for (std::vector<std::tr1::shared_ptr<rss_item> >::iterator it=items_.begin();it!=items_.end();it++) {
+		(*it)->set_feedptr(self);
+	}
+}
+
 void rss_item::set_feedptr(std::tr1::shared_ptr<rss_feed> ptr) {
 	feedptr = ptr;
 }
 
+
+}
