@@ -5,7 +5,7 @@
 
 #include <langinfo.h>
 
-using namespace newsbeuter;
+namespace newsbeuter {
 
 /*
  * This is a wrapper around the low-level C functions of STFL.
@@ -62,10 +62,33 @@ std::string stfl::form::get_focus() {
 
 void stfl::form::set_focus(const std::string& name) {
 	stfl_set_focus(f, stfl_ipool_towc(ipool,name.c_str()));
-	GetLogger().log(LOG_DEBUG,"stfl::form::set_focus: %s rc = %d", name.c_str());
+	LOG(LOG_DEBUG,"stfl::form::set_focus: %s", name.c_str());
+}
+
+void stfl::form::modify(const std::string& name, const std::string& mode, const std::string& text) {
+	const wchar_t * wname, * wmode, * wtext;
+	wname = stfl_ipool_towc(ipool,name.c_str());
+	wmode = stfl_ipool_towc(ipool,mode.c_str());
+	wtext = stfl_ipool_towc(ipool,text.c_str());
+	stfl_modify(f, wname, wmode, wtext);
 	stfl_ipool_flush(ipool);
 }
 
+void stfl::reset() {
+	stfl_reset();
+}
+
+static mutex quote_mtx;
+
+std::string stfl::quote(const std::string& text) {
+	scope_mutex lock(&quote_mtx);
+	stfl_ipool * ipool = stfl_ipool_create(nl_langinfo(CODESET));
+	std::string retval = stfl_ipool_fromwc(ipool,stfl_quote(stfl_ipool_towc(ipool,text.c_str())));
+	stfl_ipool_destroy(ipool);
+	return retval;
+}
+
+/*
 std::string stfl::form::dump(const std::string& name, const std::string& prefix, int focus) {
 	const char * text = stfl_ipool_fromwc(ipool,stfl_dump(f, stfl_ipool_towc(ipool,name.c_str()), stfl_ipool_towc(ipool,prefix.c_str()), focus));
 	std::string retval;
@@ -75,17 +98,6 @@ std::string stfl::form::dump(const std::string& name, const std::string& prefix,
 	return retval;
 }
 
-void stfl::form::modify(const std::string& name, const std::string& mode, const std::string& text) {
-	GetLogger().log(LOG_INFO, "stfl::form::modify: name = `%s' mode = `%s' text = `%s'", name.c_str(), mode.c_str(), text.c_str());
-	const wchar_t * wname, * wmode, * wtext;
-	wname = stfl_ipool_towc(ipool,name.c_str());
-	wmode = stfl_ipool_towc(ipool,mode.c_str());
-	wtext = stfl_ipool_towc(ipool,text.c_str());
-	// GetLogger().log(LOG_INFO, "stfl::form::modify: wname = `%ls' mode = `%ls' text = `%ls'", wname, wmode, wtext);
-	stfl_modify(f, wname, wmode, wtext);
-	stfl_ipool_flush(ipool);
-}
-
 std::string stfl::form::lookup(const std::string& path, const std::string& newname) {
 	const char * text = stfl_ipool_fromwc(ipool, stfl_lookup(f, stfl_ipool_towc(ipool,path.c_str()), stfl_ipool_towc(ipool,newname.c_str())));
 	std::string retval;
@@ -107,17 +119,6 @@ void stfl::error_action(const std::string& mode) {
 	stfl_error_action(stfl_ipool_towc(ipool,mode.c_str()));
 	stfl_ipool_destroy(ipool);
 }
+*/
 
-void stfl::reset() {
-	stfl_reset();
-}
-
-static mutex quote_mtx;
-
-std::string stfl::quote(const std::string& text) {
-	scope_mutex lock(&quote_mtx);
-	stfl_ipool * ipool = stfl_ipool_create(nl_langinfo(CODESET));
-	std::string retval = stfl_ipool_fromwc(ipool,stfl_quote(stfl_ipool_towc(ipool,text.c_str())));
-	stfl_ipool_destroy(ipool);
-	return retval;
 }
