@@ -117,43 +117,37 @@ void opml_urlreader::reload() {
 
 	std::string user_agent = utils::get_useragent(cfg);
 
-	std::string urlcontent = utils::retrieve_url(this->get_source(), user_agent.c_str(), this->get_auth(), cfg->get_configvalue_as_int("download-timeout"));
-	// GetLogger().log(LOG_DEBUG, "bloglines_urlreader::reload: return OPML content is `%s'", urlcontent.c_str());
+	std::vector<std::string> urls = utils::tokenize_quoted(this->get_source(), " ");
 
-	nxml_t *data;
-	nxml_data_t * root, * body;
-	nxml_error_t ret;
+	for (std::vector<std::string>::iterator it=urls.begin();it!=urls.end();it++) {
+		LOG(LOG_DEBUG, "bloglines_urlread::reload: downloading `%s'", it->c_str());
+		std::string urlcontent = utils::retrieve_url(*it, cfg);
 
-	ret = nxml_new (&data);
-	if (ret != NXML_OK) {
-		// TODO: error message
-		return;
-	}
+		xmlDoc * doc = xmlParseMemory(urlcontent.c_str(), urlcontent.length());
 
-	ret = nxml_parse_buffer(data, const_cast<char *>(urlcontent.c_str()), urlcontent.length());
-	if (ret != NXML_OK) {
-		// puts (nxml_strerror (data, ret));
-		GetLogger().log(LOG_ERROR, "opml_urlreader::reload: parsing XML file failed: %s", nxml_strerror(data, ret));
-		return;
-	}
-
-	nxml_root_element (data, &root);
-
-	if (root) {
-		body = nxmle_find_element(data, root, "body", NULL);
-		if (body) {
-			GetLogger().log(LOG_DEBUG, "opml_urlreader::reload: found body");
-			rec_find_rss_outlines(body, "");
+		if (doc == NULL) {
+			LOG(LOG_ERROR, "opml_urlreader::reload: parsing XML file failed");
+			continue;
 		}
+
+		xmlNode * root = xmlDocGetRootElement(doc);
+
+		if (root) {
+			for (xmlNode * node = root->children; node != NULL; node = node->next) {
+				if (strcmp((const char *)node->name, "body")==0) {
+					LOG(LOG_DEBUG, "opml_urlreader::reload: found body");
+					rec_find_rss_outlines(node->children, "");
+				}
+			}
+		}
+
+		xmlFreeDoc(doc);
 	}
-
-	nxml_free(data);
-
 }
 
-void opml_urlreader::handle_node(nxml_data_t * node, const std::string& tag) {
+void opml_urlreader::handle_node(xmlNode * node, const std::string& tag) {
 	if (node) {
-		const char * rssurl = nxmle_find_attribute(node, "xmlUrl", NULL);
+		char * rssurl = (char *)xmlGetProp(node, (const xmlChar *)"xmlUrl");
 		if (rssurl && strlen(rssurl) > 0) {
 			std::string theurl(rssurl);
 			urls.push_back(theurl);
@@ -164,12 +158,14 @@ void opml_urlreader::handle_node(nxml_data_t * node, const std::string& tag) {
 				alltags.insert(tag);
 			}
 		}
+		if (rssurl)
+			xmlFree(rssurl);
 	}
 }
 
-void bloglines_urlreader::handle_node(nxml_data_t * node, const std::string& tag) {
+void bloglines_urlreader::handle_node(xmlNode * node, const std::string& tag) {
 	if (node) {
-		const char * sub_id = nxmle_find_attribute(node, "BloglinesSubId", NULL);
+		char * sub_id = (char *)xmlGetProp(node, (const xmlChar *)"BloglinesSubId");
 		if (sub_id) {
 			std::string theurl = getitems_url;
 			theurl.append("?s=");
@@ -180,7 +176,7 @@ void bloglines_urlreader::handle_node(nxml_data_t * node, const std::string& tag
 			}
 
 			std::string auth = cfg->get_configvalue("bloglines-auth");
-			GetLogger().log(LOG_DEBUG, "bloglines_urlreader::rec_find_rss_outlines: auth = %s", auth.c_str());
+			LOG(LOG_DEBUG, "bloglines_urlreader::rec_find_rss_outlines: auth = %s", auth.c_str());
 			auth = utils::replace_all(auth,"@","%40");
 
 			if (theurl.substr(0,7) == "http://") {
@@ -196,29 +192,33 @@ void bloglines_urlreader::handle_node(nxml_data_t * node, const std::string& tag
 				tags[theurl] = tmptags;
 				alltags.insert(tag);
 			}
+			xmlFree(sub_id);
 		}
 	}
 }
 
-void opml_urlreader::rec_find_rss_outlines(nxml_data_t * node, std::string tag) {
+void opml_urlreader::rec_find_rss_outlines(xmlNode * node, std::string tag) {
 	while (node) {
-		const char * type = nxmle_find_attribute(node, "type", NULL);
+		char * type = (char *)xmlGetProp(node, (const xmlChar *)"type");
 
 		std::string newtag = tag;
 
-		if (node->type == NXML_TYPE_ELEMENT && strcmp(node->value,"outline")==0) {
+		if (strcmp((const char *)node->name, "outline")==0) {
 			if (type && strcmp(type,"rss")==0) {
 				handle_node(node, tag);
 			} else {
-				const char * text = nxmle_find_attribute(node, "title", NULL);
+				char * text = (char *)xmlGetProp(node, (const xmlChar *)"title");
 				if (text) {
 					if (newtag.length() > 0) {
 						newtag.append("/");
 					}
 					newtag.append(text);
+					xmlFree(text);
 				}
 			}
 		}
+		xmlFree(type);
+
 		rec_find_rss_outlines(node->children, newtag);
 
 		node = node->next;
