@@ -22,6 +22,7 @@
 
 #include <langinfo.h>
 #include <stfl.h>
+#include <libxml/uri.h>
 
 namespace newsbeuter {
 
@@ -118,8 +119,6 @@ std::vector<std::string> utils::tokenize_quoted(const std::string& str, std::str
 		last_pos = str.find_first_not_of(delimiters, pos);
 	}
 
-	GetLogger().log(LOG_DEBUG, "utils::tokenize_quoted: tokenizing '%s' resulted in %u elements", str.c_str(), tokens.size());
-
 	return tokens;
 }
 
@@ -182,7 +181,7 @@ std::vector<std::string> utils::tokenize_nl(const std::string& str, std::string
 	std::string::size_type pos = str.find_first_of(delimiters, last_pos);
 	unsigned int i;
 
-	GetLogger().log(LOG_DEBUG,"utils::tokenize_nl: last_pos = %u",last_pos);
+	LOG(LOG_DEBUG,"utils::tokenize_nl: last_pos = %u",last_pos);
 	if (last_pos != std::string::npos) {
 		for (i=0;i<last_pos;++i) {
 			tokens.push_back(std::string("\n"));
@@ -191,9 +190,9 @@ std::vector<std::string> utils::tokenize_nl(const std::string& str, std::string
 
 	while (std::string::npos != pos || std::string::npos != last_pos) {
 		tokens.push_back(str.substr(last_pos, pos - last_pos));
-		GetLogger().log(LOG_DEBUG,"utils::tokenize_nl: substr = %s", str.substr(last_pos, pos - last_pos).c_str());
+		LOG(LOG_DEBUG,"utils::tokenize_nl: substr = %s", str.substr(last_pos, pos - last_pos).c_str());
 		last_pos = str.find_first_not_of(delimiters, pos);
-		GetLogger().log(LOG_DEBUG,"utils::tokenize_nl: pos - last_pos = %u", last_pos - pos);
+		LOG(LOG_DEBUG,"utils::tokenize_nl: pos - last_pos = %u", last_pos - pos);
 		for (i=0;last_pos != std::string::npos && pos != std::string::npos && i<(last_pos - pos);++i) {
 			tokens.push_back(std::string("\n"));
 		}
@@ -204,7 +203,7 @@ std::vector<std::string> utils::tokenize_nl(const std::string& str, std::string
 }
 
 void utils::remove_fs_lock(const std::string& lock_file) {
-	GetLogger().log(LOG_DEBUG, "utils::remove_fs_lock: removed lockfile %s", lock_file.c_str());
+	LOG(LOG_DEBUG, "utils::remove_fs_lock: removed lockfile %s", lock_file.c_str());
 	::unlink(lock_file.c_str());
 }
 
@@ -213,7 +212,7 @@ bool utils::try_fs_lock(const std::string& lock_file, pid_t & pid) {
 	// pid == 0 indicates that something went majorly wrong during locking
 	pid = 0;
 
-	GetLogger().log(LOG_DEBUG, "utils::try_fs_lock: trying to lock %s", lock_file.c_str());
+	LOG(LOG_DEBUG, "utils::try_fs_lock: trying to lock %s", lock_file.c_str());
 
 	// first, we open (and possibly create) the lock file
 	fd = ::open(lock_file.c_str(), O_RDWR | O_CREAT, 0600);
@@ -222,11 +221,10 @@ bool utils::try_fs_lock(const std::string& lock_file, pid_t & pid) {
 
 	// then we lock it (T_LOCK returns immediately if locking is not possible)
 	if (lockf(fd, F_TLOCK, 0) == 0) {
-		char buf[32];
-		snprintf(buf, sizeof(buf), "%u", getpid());
+		std::string pidtext = utils::to_s(getpid());
 		// locking successful -> truncate file and write own PID into it
 		ftruncate(fd, 0);
-		write(fd, buf, strlen(buf));
+		write(fd, pidtext.c_str(), pidtext.length());
 		return true;
 	}
 
@@ -235,7 +233,7 @@ bool utils::try_fs_lock(const std::string& lock_file, pid_t & pid) {
 	if (fd >= 0) {
 		char buf[32];
 		int len = read(fd, buf, sizeof(buf)-1);
-		unsigned int upid;
+		unsigned int upid = 0;
 		buf[len] = '\0';
 		sscanf(buf, "%u", &upid);
 		pid = upid;
@@ -248,7 +246,7 @@ bool utils::try_fs_lock(const std::string& lock_file, pid_t & pid) {
 std::string utils::convert_text(const std::string& text, const std::string& tocode, const std::string& fromcode) {
 	std::string result;
 
-	if (tocode == fromcode)
+	if (strcasecmp(tocode.c_str(), fromcode.c_str())==0)
 		return text;
 
 	iconv_t cd = ::iconv_open((tocode + "//TRANSLIT").c_str(), fromcode.c_str());
@@ -294,6 +292,8 @@ std::string utils::convert_text(const std::string& text, const std::string& toco
 					inbufp += strlen(inbufp) - inbytesleft + 1;
 					inbytesleft = strlen(inbufp);
 					break;
+				default:
+					break;
 			}
 		} else {
 			result.append(old_outbufp, outbufp - old_outbufp);
@@ -325,7 +325,7 @@ void utils::extract_filter(const std::string& line, std::string& filter, std::st
 	filter = line.substr(pos+1, pos1 - pos - 1);
 	pos = pos1;
 	url = line.substr(pos+1, line.length() - pos);
-	GetLogger().log(LOG_DEBUG, "utils::extract_filter: %s -> filter: %s url: %s", line.c_str(), filter.c_str(), url.c_str());
+	LOG(LOG_DEBUG, "utils::extract_filter: %s -> filter: %s url: %s", line.c_str(), filter.c_str(), url.c_str());
 }
 
 static size_t my_write_data(void *buffer, size_t size, size_t nmemb, void *userp) {
@@ -334,29 +334,19 @@ static size_t my_write_data(void *buffer, size_t size, size_t nmemb, void *userp
 	return size * nmemb;
 }
 
-std::string utils::retrieve_url(const std::string& url, const char * user_agent, const char * auth, int download_timeout) {
+std::string utils::retrieve_url(const std::string& url, configcontainer * cfgcont) {
 	std::string buf;
 
 	CURL * easyhandle = curl_easy_init();
-	if (user_agent) {
-		curl_easy_setopt(easyhandle, CURLOPT_USERAGENT, user_agent);
-	}
+	set_common_curl_options(easyhandle, cfgcont);
 	curl_easy_setopt(easyhandle, CURLOPT_URL, url.c_str());
 	curl_easy_setopt(easyhandle, CURLOPT_WRITEFUNCTION, my_write_data);
 	curl_easy_setopt(easyhandle, CURLOPT_WRITEDATA, &buf);
-	curl_easy_setopt(easyhandle, CURLOPT_NOSIGNAL, 1);
-	curl_easy_setopt(easyhandle, CURLOPT_ENCODING, "gzip, deflate");
-	curl_easy_setopt(easyhandle, CURLOPT_TIMEOUT, download_timeout);
-
-	if (auth) {
-		curl_easy_setopt(easyhandle, CURLOPT_USERPWD, auth);
-		curl_easy_setopt(easyhandle, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
-	}
 
 	curl_easy_perform(easyhandle);
 	curl_easy_cleanup(easyhandle);
 
-	GetLogger().log(LOG_DEBUG, "utils::retrieve_url(%s): %s", url.c_str(), buf.c_str());
+	LOG(LOG_DEBUG, "utils::retrieve_url(%s): %s", url.c_str(), buf.c_str());
 
 	return buf;
 }
@@ -373,9 +363,9 @@ void utils::run_command(const std::string& cmd, const std::string& input) {
 			dup2(fd, 0);
 			dup2(fd, 1);
 			dup2(fd, 2);
-			GetLogger().log(LOG_DEBUG, "utils::run_command: %s '%s'", cmd.c_str(), input.c_str());
+			LOG(LOG_DEBUG, "utils::run_command: %s '%s'", cmd.c_str(), input.c_str());
 			execlp(cmd.c_str(), cmd.c_str(), input.c_str(), NULL);
-			GetLogger().log(LOG_DEBUG, "utils::run_command: execlp of %s failed: %s", cmd.c_str(), strerror(errno));
+			LOG(LOG_DEBUG, "utils::run_command: execlp of %s failed: %s", cmd.c_str(), strerror(errno));
 			exit(1);
 		}
 		default:
@@ -450,13 +440,11 @@ std::string utils::resolve_tilde(const std::string& str) {
 }
 
 std::string utils::replace_all(std::string str, const std::string& from, const std::string& to) {
-	GetLogger().log(LOG_DEBUG,"utils::replace_all: before str = %s", str.c_str());
 	std::string::size_type s = str.find(from);
 	while (s != std::string::npos) {
 		str.replace(s,from.length(), to);
 		s = str.find(from, s + to.length());
 	}
-	GetLogger().log(LOG_DEBUG,"utils::replace_all: after str = %s", str.c_str());
 	return str;
 }
 
@@ -484,41 +472,23 @@ std::string utils::wstr2str(const std::wstring& wstr) {
 }
 
 std::string utils::to_s(unsigned int u) {
-	std::ostringstream os;
-	os << u;
-	return os.str();
+	char buf[32];
+	snprintf(buf, sizeof(buf), "%u", u);
+	return buf;
 }
 
 std::string utils::absolute_url(const std::string& url, const std::string& link) {
-	if (link.substr(0,7)=="http://" || link.substr(0,8)=="https://" || link.substr(0,6)=="ftp://" || link.substr(0,7) == "mailto:"){
-		return link;
-	}
-	char u[1024];
-	snprintf(u, sizeof(u), "%s", url.c_str());
-	if (link[0] == '/') {
-		// this is probably the worst code in the whole program
-		char * foo = strstr(u, "//");
-		if (foo) {
-			if (strlen(foo)>=2) {
-				foo += 2;
-				foo = strchr(foo,'/');
-				char u2[1024];
-				strcpy(u2, u);
-				snprintf(u2 + (foo - u), sizeof(u2) - (foo - u), "%s", link.c_str());
-				return u2;
-			}
-		}
-		return link;
+	xmlChar * newurl = xmlBuildURI((const xmlChar *)link.c_str(), (const xmlChar *)url.c_str());
+	std::string retval;
+	if (newurl) {
+		retval = (const char *)newurl;
+		xmlFree(newurl);
 	} else {
-		char * base = dirname(u);
-		std::string retval(base);
-		retval.append(1,'/');
-		retval.append(link);
-		return retval;
+		retval = link;
 	}
+	return retval;
 }
 
-
 std::string utils::strprintf(const char * format, ...) {
 	if (!format)
 		return std::string("");
@@ -553,7 +523,7 @@ std::string utils::get_useragent(configcontainer * cfgcont) {
 
 unsigned int utils::to_u(const std::string& str) {
 	std::istringstream is(str);
-	unsigned int u;
+	unsigned int u = 0;
 	is >> u;
 	return u;
 }
@@ -566,13 +536,13 @@ scope_measure::scope_measure(const std::string& func, loglevel ll) : lvl(ll) {
 void scope_measure::stopover(const std::string& son) {
 	gettimeofday(&tv2, NULL);
 	unsigned long diff = (((tv2.tv_sec - tv1.tv_sec) * 1000000) + tv2.tv_usec) - tv1.tv_usec;
-	GetLogger().log(lvl, "scope_measure: function `%s' (stop over `%s') took %lu.%06lu s so far", funcname.c_str(), son.c_str(), diff / 1000000, diff % 1000000);
+	LOG(lvl, "scope_measure: function `%s' (stop over `%s') took %lu.%06lu s so far", funcname.c_str(), son.c_str(), diff / 1000000, diff % 1000000);
 }
 
 scope_measure::~scope_measure() {
 	gettimeofday(&tv2, NULL);
 	unsigned long diff = (((tv2.tv_sec - tv1.tv_sec) * 1000000) + tv2.tv_usec) - tv1.tv_usec;
-	GetLogger().log(LOG_INFO, "scope_measure: function `%s' took %lu.%06lu s", funcname.c_str(), diff / 1000000, diff % 1000000);
+	LOG(LOG_INFO, "scope_measure: function `%s' took %lu.%06lu s", funcname.c_str(), diff / 1000000, diff % 1000000);
 }
 
 void utils::append_escapes(std::string& str, char c) {
@@ -623,4 +593,124 @@ size_t utils::strwidth(const std::string& str) {
 	return wcswidth(wstr.c_str(), wstr.length());
 }
 
+std::string utils::join(const std::vector<std::string>& strings, const std::string& separator) {
+	std::string result;
+
+	for (std::vector<std::string>::const_iterator it=strings.begin();it!=strings.end();it++) {
+		result.append(*it);
+		result.append(separator);
+	}
+
+	if (result.length() > 0)
+		result.erase(result.length()-separator.length(), result.length());
+
+	return result;
+}
+
+std::string utils::censor_url(const std::string& url) {
+	std::string rv;
+	if (url.length() > 0) {
+		const char * myuri = url.c_str();
+		xmlURIPtr uri = xmlParseURI(myuri);
+		if (uri) {
+			if (uri->user) {
+				xmlFree(uri->user);
+				uri->user = (char *)xmlStrdup((const xmlChar *)"*:*");
+			}
+			xmlChar * uristr = xmlSaveUri(uri);
+			
+			rv = (const char *)uristr;
+			xmlFree(uristr);
+			xmlFreeURI(uri);
+		} else
+			return url;
+	}
+	return rv;
+}
+
+std::string utils::quote_for_stfl(std::string str) {
+	unsigned int len = str.length();
+	for (unsigned int i=0;i<len;++i) {
+		if (str[i] == '<') {
+			str.insert(i+1, ">");
+			++len;
+		}
+	}
+	return str;
+}
+
+void utils::trim(std::string& str) {
+	while (str.length() > 0 && (str[0] == ' ' || str[0] == '\n' || str[0] == '\r')) {
+		str.erase(0,1);
+	}
+	trim_end(str);
+}
+
+void utils::trim_end(std::string& str) {
+	std::string::size_type pos = str.length()-1;
+	while (str.length()>0 && (str[pos] == '\n' || str[pos] == '\r')) {
+		str.erase(pos);
+		pos--;
+	}
+}
+
+std::string utils::quote(const std::string& str) {
+	std::string rv = replace_all(str, "\"", "\\\"");
+	rv.insert(0, "\"");
+	rv.append("\"");
+	return rv;
+}
+
+unsigned int utils::get_random_value(unsigned int max) {
+	static bool initialized = false;
+	unsigned int var;
+	if (!initialized) {
+		initialized = true;
+		srand(~(time(NULL) ^ getpid() ^ getppid() ^ var));
+	}
+	return static_cast<unsigned int>(rand() % max);
+}
+
+std::string utils::quote_if_necessary(const std::string& str) {
+	std::string result;
+	if (str.find_first_of(" ", 0) == std::string::npos) {
+		result = str;
+	} else {
+		result = utils::replace_all(str, "\"", "\\\"");
+		result.insert(0, "\"");
+		result.append("\"");
+	}
+	return result;
+}
+
+void utils::set_common_curl_options(CURL * handle, configcontainer * cfg) {
+	std::string proxy;
+	std::string proxyauth;
+	std::string useragent; 
+	unsigned int dl_timeout = 0;
+
+
+	if (cfg) {
+		proxy = cfg->get_configvalue("proxy");
+		proxyauth = cfg->get_configvalue("proxy-auth");
+		useragent = utils::get_useragent(cfg);
+		dl_timeout = cfg->get_configvalue_as_int("download-timeout");
+	}
+
+	curl_easy_setopt(handle, CURLOPT_SSL_VERIFYPEER, 0);
+	curl_easy_setopt(handle, CURLOPT_NOSIGNAL, 1);
+	curl_easy_setopt(handle, CURLOPT_ENCODING, "gzip, deflate");
+	curl_easy_setopt(handle, CURLOPT_TIMEOUT, dl_timeout);
+
+	if (proxy != "")
+		curl_easy_setopt(handle, CURLOPT_PROXY, proxy.c_str());
+	if (proxyauth != "")
+		curl_easy_setopt(handle, CURLOPT_PROXYUSERPWD, proxyauth.c_str());
+	curl_easy_setopt(handle, CURLOPT_USERAGENT, useragent.c_str());
+
+	curl_easy_setopt(handle, CURLOPT_FOLLOWLOCATION, 1);
+	curl_easy_setopt(handle, CURLOPT_MAXREDIRS, 10);
+	curl_easy_setopt(handle, CURLOPT_FAILONERROR, 1);
+}
+
 }
