@@ -65,11 +65,6 @@ BOOST_AUTO_TEST_CASE(TestNewsbeuterReload) {
 	BOOST_CHECK_EQUAL(feed2->items()[0]->title(), "Another Title");
 	BOOST_CHECK_EQUAL(feed2->items()[7]->title(), "Handy als IR-Detektor");
 
-	rsscache->set_lastmodified("http://bereshit.synflood.at/~ak/rss.xml", 1000);
-	BOOST_CHECK_EQUAL(rsscache->get_lastmodified("http://bereshit.synflood.at/~ak/rss.xml"), 1000);
-	rsscache->set_lastmodified("http://bereshit.synflood.at/~ak/rss.xml", 0);
-	BOOST_CHECK_EQUAL(rsscache->get_lastmodified("http://bereshit.synflood.at/~ak/rss.xml"), 1000);
-
 	std::vector<std::string> feedurls = rsscache->get_feed_urls();
 	BOOST_CHECK_EQUAL(feedurls.size(), 1u);
 	BOOST_CHECK_EQUAL(feedurls[0], "http://bereshit.synflood.at/~ak/rss.xml");
@@ -96,7 +91,7 @@ BOOST_AUTO_TEST_CASE(TestConfigParserContainerAndKeymap) {
 	try {
 		cfgparser.parse("test-config.txt");
 	}  catch (const configexception& ex) {
-		GetLogger().log(LOG_ERROR,"an exception occured while parsing the configuration file: %s", ex.what());
+		LOG(LOG_ERROR,"an exception occured while parsing the configuration file: %s", ex.what());
 		BOOST_CHECK(false);
 	} catch (...) {
 		BOOST_CHECK(false);
@@ -146,15 +141,43 @@ BOOST_AUTO_TEST_CASE(TestConfigParserContainerAndKeymap) {
 	BOOST_CHECK_EQUAL(k.get_key("^A"), '\001');
 
 	std::vector<std::string> params;
-	BOOST_CHECK_EQUAL(k.handle_action("bind-key", params), AHS_TOO_FEW_PARAMS);
-	BOOST_CHECK_EQUAL(k.handle_action("unbind-key", params), AHS_TOO_FEW_PARAMS);
-	BOOST_CHECK_EQUAL(k.handle_action("macro", params), AHS_TOO_FEW_PARAMS);
+	try {
+		k.handle_action("bind-key", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK(std::string(e.what()).length() > 0);
+	}
+	try {
+		k.handle_action("unbind-key", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK(std::string(e.what()).length() > 0);
+	}
+	try {
+		k.handle_action("macro", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK(std::string(e.what()).length() > 0);
+	}
 	params.push_back("r");
-	BOOST_CHECK_EQUAL(k.handle_action("bind-key", params), AHS_TOO_FEW_PARAMS);
-	BOOST_CHECK_EQUAL(k.handle_action("unbind-key", params), AHS_OK);
+	try {
+		k.handle_action("bind-key", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK(std::string(e.what()).length() > 0);
+	}
+	try {
+		k.handle_action("unbind-key", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK_EQUAL(std::string(e.what()).length(), 0u);
+	}
 	params.push_back("open");
-	BOOST_CHECK_EQUAL(k.handle_action("bind-key", params), AHS_OK);
-	BOOST_CHECK_EQUAL(k.handle_action("an-invalid-action", params), AHS_INVALID_PARAMS);
+	try {
+		k.handle_action("bind-key", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK_EQUAL(std::string(e.what()).length(), 0u);
+	}
+	try {
+		k.handle_action("an-invalid-action", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK(std::string(e.what()).length() > 0);
+	}
 }
 
 BOOST_AUTO_TEST_CASE(TestTagSoupPullParser) {
@@ -300,18 +323,18 @@ BOOST_AUTO_TEST_CASE(TestFilterLanguage) {
 	FilterParser fp;
 
 	// test parser
-	BOOST_CHECK_EQUAL(fp.parse_string("a = \"b\""), true);
-	BOOST_CHECK_EQUAL(fp.parse_string("a = \"b"), false);
-	BOOST_CHECK_EQUAL(fp.parse_string("a = b"), false);
-	BOOST_CHECK_EQUAL(fp.parse_string("(a=\"b\")"), true);
-	BOOST_CHECK_EQUAL(fp.parse_string("((a=\"b\"))"), true);
-	BOOST_CHECK_EQUAL(fp.parse_string("((a=\"b\")))"), false);
+	BOOST_CHECK_MESSAGE(fp.parse_string("a = \"b\""), utils::wstr2str(fp.get_error()).c_str());
+	BOOST_CHECK_MESSAGE(!fp.parse_string("a = \"b"), utils::wstr2str(fp.get_error()).c_str());
+	BOOST_CHECK_MESSAGE(!fp.parse_string("a = b"), utils::wstr2str(fp.get_error()).c_str());
+	BOOST_CHECK_MESSAGE(fp.parse_string("(a=\"b\")"), utils::wstr2str(fp.get_error()).c_str());
+	BOOST_CHECK_MESSAGE(fp.parse_string("((a=\"b\"))"), utils::wstr2str(fp.get_error()).c_str());
+	BOOST_CHECK_MESSAGE(!fp.parse_string("((a=\"b\")))"), utils::wstr2str(fp.get_error()).c_str());
 
 	// test operators
-	BOOST_CHECK_EQUAL(fp.parse_string("a != \"b\""), true);
-	BOOST_CHECK_EQUAL(fp.parse_string("a =~ \"b\""), true);
-	BOOST_CHECK_EQUAL(fp.parse_string("a !~ \"b\""), true);
-	BOOST_CHECK_EQUAL(fp.parse_string("a !! \"b\""), false);
+	BOOST_CHECK_MESSAGE(fp.parse_string("a != \"b\""), utils::wstr2str(fp.get_error()).c_str());
+	BOOST_CHECK_MESSAGE(fp.parse_string("a =~ \"b\""), utils::wstr2str(fp.get_error()).c_str());
+	BOOST_CHECK_MESSAGE(fp.parse_string("a !~ \"b\""), utils::wstr2str(fp.get_error()).c_str());
+	BOOST_CHECK_MESSAGE(!fp.parse_string("a !! \"b\""), utils::wstr2str(fp.get_error()).c_str());
 
 	// complex query
 	BOOST_CHECK_EQUAL(fp.parse_string("( a = \"b\") and ( b = \"c\" ) or ( ( c != \"d\" ) and ( c !~ \"asdf\" )) or c != \"xx\""), true);
@@ -493,6 +516,7 @@ BOOST_AUTO_TEST_CASE(TestMiscUtilsFunctions) {
 	BOOST_CHECK_EQUAL(utils::replace_all("", "a", "b"), "");
 	BOOST_CHECK_EQUAL(utils::replace_all("aaaa", "b", "c"), "aaaa");
 	BOOST_CHECK_EQUAL(utils::replace_all("this is a normal test text", " t", " T"), "this is a normal Test Text");
+	BOOST_CHECK_EQUAL(utils::replace_all("o o o", "o", "<o>"), "<o> <o> <o>");
 
 	BOOST_CHECK_EQUAL(utils::to_s(0), "0");
 	BOOST_CHECK_EQUAL(utils::to_s(100), "100");
@@ -516,13 +540,21 @@ BOOST_AUTO_TEST_CASE(TestRegexManager) {
 	regexmanager rxman;
 
 	std::vector<std::string> params;
-	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_TOO_FEW_PARAMS);
+	try {
+		rxman.handle_action("highlight", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK(std::string(e.what()).length() > 0);
+	}
 	params.push_back("articlelist");
 	params.push_back("foo");
 	params.push_back("blue");
 	params.push_back("red");
 
-	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_OK);
+	try {
+		rxman.handle_action("highlight", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK_EQUAL(std::string(e.what()).length(), 0u);
+	}
 
 	std::string str = "xfoox";
 	rxman.quote_and_highlight(str, "articlelist");
@@ -533,36 +565,60 @@ BOOST_AUTO_TEST_CASE(TestRegexManager) {
 	BOOST_CHECK_EQUAL(str, "xfoox");
 
 	params[0] = "feedlist";
-	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_OK);
+	try {
+		rxman.handle_action("highlight", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK_EQUAL(std::string(e.what()).length(), 0u);
+	}
 
 	str = "yfooy";
 	rxman.quote_and_highlight(str, "feedlist");
 	BOOST_CHECK_EQUAL(str, "y<0>foo</>y");
 
 	params[0] = "invalidloc";
-	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_INVALID_PARAMS);
+	try {
+		rxman.handle_action("highlight", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK(std::string(e.what()).length() > 0);
+	}
 
 	params[0] = "feedlist";
 	params[1] = "*";
-	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_INVALID_PARAMS);
+	try {
+		rxman.handle_action("highlight", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK(std::string(e.what()).length() > 0);
+	}
 
 	params[1] = "foo";
 	params.push_back("bold");
 	params.push_back("underline");
-	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_OK);
+	try {
+		rxman.handle_action("highlight", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK_EQUAL(std::string(e.what()).length(), 0u);
+	}
 
 	params[0] = "all";
-	BOOST_CHECK_EQUAL(rxman.handle_action("highlight", params), AHS_OK);
+	try {
+		rxman.handle_action("highlight", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK_EQUAL(std::string(e.what()).length(), 0u);
+	}
 
-	BOOST_CHECK_EQUAL(rxman.handle_action("invalidcommand", params), AHS_INVALID_COMMAND);
+	try {
+		rxman.handle_action("an-invalid-command", params);
+	} catch (const confighandlerexception& e) {
+		BOOST_CHECK(std::string(e.what()).length() > 0);
+	}
 
 	str = "<";
 	rxman.quote_and_highlight(str, "feedlist");
-	BOOST_CHECK_EQUAL(str, "<>");
+	BOOST_CHECK_EQUAL(str, "<");
 
 	str = "a<b>";
 	rxman.quote_and_highlight(str, "feedlist");
-	BOOST_CHECK_EQUAL(str, "a<>b>");
+	BOOST_CHECK_EQUAL(str, "a<b>");
 }
 BOOST_AUTO_TEST_CASE(TestHtmlRenderer) {
 	htmlrenderer rnd(100);
@@ -572,7 +628,7 @@ BOOST_AUTO_TEST_CASE(TestHtmlRenderer) {
 
 	rnd.render("<a href=\"http://slashdot.org/\">slashdot</a>", lines, links, "");
 	BOOST_CHECK(lines.size() >= 1);
-	BOOST_CHECK_EQUAL(lines[0], "[slashdot][1]");
+	BOOST_CHECK_EQUAL(lines[0], "<u>slashdot</>[1]");
 	BOOST_CHECK_EQUAL(links[0].first, "http://slashdot.org/");
 	BOOST_CHECK_EQUAL(links[0].second, LINK_HREF);
 
@@ -633,3 +689,99 @@ BOOST_AUTO_TEST_CASE(TestIndexPartitioning) {
 	BOOST_CHECK_EQUAL(partitions[0].first, 0u);
 	BOOST_CHECK_EQUAL(partitions[0].second, 103u);
 }
+
+BOOST_AUTO_TEST_CASE(TestCensorUrl) {
+	BOOST_CHECK_EQUAL(utils::censor_url(""), "");
+	BOOST_CHECK_EQUAL(utils::censor_url("foobar"), "foobar");
+	BOOST_CHECK_EQUAL(utils::censor_url("foobar://xyz/"), "foobar://xyz/");
+
+	BOOST_CHECK_EQUAL(utils::censor_url("http://newsbeuter.org/"), "http://newsbeuter.org/");
+	BOOST_CHECK_EQUAL(utils::censor_url("https://newsbeuter.org/"), "https://newsbeuter.org/");
+
+	BOOST_CHECK_EQUAL(utils::censor_url("http://@newsbeuter.org/"), "http://*:*@newsbeuter.org/");
+	BOOST_CHECK_EQUAL(utils::censor_url("https://@newsbeuter.org/"), "https://*:*@newsbeuter.org/");
+
+	BOOST_CHECK_EQUAL(utils::censor_url("http://foo:bar@newsbeuter.org/"), "http://*:*@newsbeuter.org/");
+	BOOST_CHECK_EQUAL(utils::censor_url("https://foo:bar@newsbeuter.org/"), "https://*:*@newsbeuter.org/");
+
+	BOOST_CHECK_EQUAL(utils::censor_url("http://aschas@newsbeuter.org/"), "http://*:*@newsbeuter.org/");
+	BOOST_CHECK_EQUAL(utils::censor_url("https://aschas@newsbeuter.org/"), "https://*:*@newsbeuter.org/");
+
+	BOOST_CHECK_EQUAL(utils::censor_url("xxx://aschas@newsbeuter.org/"), "xxx://*:*@newsbeuter.org/");
+
+	BOOST_CHECK_EQUAL(utils::censor_url("http://foobar"), "http://foobar");
+	BOOST_CHECK_EQUAL(utils::censor_url("https://foobar"), "https://foobar");
+
+	BOOST_CHECK_EQUAL(utils::censor_url("http://aschas@host"), "http://*:*@host");
+	BOOST_CHECK_EQUAL(utils::censor_url("https://aschas@host"), "https://*:*@host");
+
+	BOOST_CHECK_EQUAL(utils::censor_url("query:name:age between 1:10"), "query:name:age between 1:10");
+}
+
+BOOST_AUTO_TEST_CASE(TestMakeAbsoluteUrl) {
+	BOOST_CHECK_EQUAL(utils::absolute_url("http://foobar/hello/crook/", "bar.html"), "http://foobar/hello/crook/bar.html");
+	BOOST_CHECK_EQUAL(utils::absolute_url("https://foobar/foo/", "/bar.html"), "https://foobar/bar.html");
+	BOOST_CHECK_EQUAL(utils::absolute_url("https://foobar/foo/", "http://quux/bar.html"), "http://quux/bar.html");
+	BOOST_CHECK_EQUAL(utils::absolute_url("http://foobar", "bla.html"), "http://foobar/bla.html");
+	BOOST_CHECK_EQUAL(utils::absolute_url("http://test:test@foobar:33", "bla2.html"), "http://test:test@foobar:33/bla2.html");
+}
+
+BOOST_AUTO_TEST_CASE(TestBacktickEvaluation) {
+	BOOST_CHECK_EQUAL(configparser::evaluate_backticks(""), "");
+	BOOST_CHECK_EQUAL(configparser::evaluate_backticks("hello world"), "hello world");
+	BOOST_CHECK_EQUAL(configparser::evaluate_backticks("foo`true`baz"), "foobaz");
+	BOOST_CHECK_EQUAL(configparser::evaluate_backticks("foo`barbaz"), "foo`barbaz");
+	BOOST_CHECK_EQUAL(configparser::evaluate_backticks("foo `true` baz"), "foo  baz");
+	BOOST_CHECK_EQUAL(configparser::evaluate_backticks("foo `true` baz `xxx"), "foo  baz `xxx");
+	BOOST_CHECK_EQUAL(configparser::evaluate_backticks("`echo hello world`"), "hello world");
+	BOOST_CHECK_EQUAL(configparser::evaluate_backticks("xxx`echo yyy`zzz"), "xxxyyyzzz");
+	BOOST_CHECK_EQUAL(configparser::evaluate_backticks("`echo 3 \\* 4 | bc`"), "12");
+}
+
+BOOST_AUTO_TEST_CASE(TestUtilsQuote) {
+	BOOST_CHECK_EQUAL(utils::quote(""), "\"\"");
+	BOOST_CHECK_EQUAL(utils::quote("hello world"), "\"hello world\"");
+	BOOST_CHECK_EQUAL(utils::quote("\"hello world\""), "\"\\\"hello world\\\"\"");
+}
+
+BOOST_AUTO_TEST_CASE(TestUtilsFunctions_to_u) {
+	BOOST_CHECK_EQUAL(utils::to_u("0"), 0u);
+	BOOST_CHECK_EQUAL(utils::to_u("23"), 23u);
+	BOOST_CHECK_EQUAL(utils::to_u(""), 0u);
+}
+
+BOOST_AUTO_TEST_CASE(TestUtilsFunctions_strwidth) {
+	BOOST_CHECK_EQUAL(utils::strwidth(""), 0u);
+	BOOST_CHECK_EQUAL(utils::strwidth("xx"), 2u);
+	BOOST_CHECK_EQUAL(utils::strwidth(utils::wstr2str(L"\uF91F")), 2u);
+}
+
+BOOST_AUTO_TEST_CASE(TestUtilsFunction_join) {
+	std::vector<std::string> str;
+	BOOST_CHECK_EQUAL(utils::join(str, ""), "");
+	BOOST_CHECK_EQUAL(utils::join(str, "-"), "");
+	str.push_back("foobar");
+	BOOST_CHECK_EQUAL(utils::join(str, ""), "foobar");
+	BOOST_CHECK_EQUAL(utils::join(str, "-"), "foobar");
+	str.push_back("quux");
+	BOOST_CHECK_EQUAL(utils::join(str, ""), "foobarquux");
+	BOOST_CHECK_EQUAL(utils::join(str, "-"), "foobar-quux");
+}
+
+BOOST_AUTO_TEST_CASE(TestUtilsFunction_trim) {
+	std::string str = "  xxx\r\n";
+	utils::trim(str);
+	BOOST_CHECK_EQUAL(str, "xxx");
+	str = "\n\n abc  foobar\n";
+	utils::trim(str);
+	BOOST_CHECK_EQUAL(str, "abc  foobar");
+	str = "";
+	utils::trim(str);
+	BOOST_CHECK_EQUAL(str, "");
+	str = "     \n";
+	utils::trim(str);
+	BOOST_CHECK_EQUAL(str, "");
+	str = "quux\n";
+	utils::trim_end(str);
+	BOOST_CHECK_EQUAL(str, "quux");
+}
